Package: ascent
Title: Multi-Layer Decomposition of Functional Community Restructuring
Version: 0.1.1
Authors@R: c(
    person("Rogelio R.", "Muñoz-Li", email = "munozrogelio16@gmail.com", role = c("aut", "cre")),
    person("Flavia", "Alvarez-Denis", role = "aut")
    )
Description: Implements the 'ASC-CFD' (Assemblage Shift Characterization -
    Community Functional Dynamics) framework for decomposing functional community
    restructuring into positional (centroid displacement), dispersive
    (functional dispersion), and boundary (convex hull volume) components.
    Provides hierarchical null models (structural, quantitative, identity)
    to evaluate statistical significance and species-level leverage analysis
    to identify taxa driving functional shifts. Supports both temporal
    (paired) and spatial (pairwise) comparisons.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: cluster, ggplot2, stats, utils, vegan, geometry
Suggests: knitr, patchwork, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: https://github.com/V3ndetta96/ascent
BugReports: https://github.com/V3ndetta96/ascent/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-21 17:16:32 UTC; Usuario
Author: Rogelio R. Muñoz-Li [aut, cre],
  Flavia Alvarez-Denis [aut]
Maintainer: Rogelio R. Muñoz-Li <munozrogelio16@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-30 17:10:36 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,ascfcd)
S3method(plot,ascfcd_entities)
S3method(plot,ascfcd_pw)
S3method(plot,ascfcd_transitions)
S3method(print,ascfcd)
S3method(print,ascfcd_base)
S3method(print,ascfcd_entities)
S3method(print,ascfcd_pw)
S3method(print,ascfcd_space)
S3method(print,ascfcd_transitions)
S3method(summary,ascfcd)
S3method(summary,ascfcd_base)
S3method(summary,ascfcd_entities)
S3method(summary,ascfcd_pw)
S3method(summary,ascfcd_space)
S3method(summary,ascfcd_transitions)
export(asc_baseline)
export(asc_entities)
export(asc_null)
export(asc_paired)
export(asc_pairwise)
export(asc_transitions)
export(assess_functional_space)
import(ggplot2)
importFrom(stats,sd)
importFrom(stats,simulate)
importFrom(vegan,nullmodel)

# ascent: A multi-layer framework for decomposing functional community restructuring into positional, dispersive and boundary components under hierarchical null models.

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.20388186.svg)](https://doi.org/10.5281/zenodo.20388186)

The **`ascent`** package implements the **ASC-CFD** (Assemblage Shift Characterization — Community Functional Dynamics) framework.

Most functional diversity metrics quantify how much communities differ,
but provide little information about how those differences are organized
within functional space.

Communities may exhibit identical levels of functional turnover while
following completely different ecological trajectories, including
directional shifts, internal reorganization, or expansion and contraction
of functional boundaries.

The `ascent` package implements the ASC-CFD framework to explicitly
decompose these processes. 

`ascent` provides a multi-layer topological approach to **decompose functional community restructuring into positional, dispersive, and boundary components under hierarchical null models.**

Whether you are evaluating temporal perturbations (e.g., deforestation, climate change) or spatial beta-diversity gradients, `ascent` allows you to reconstruct the geometric trajectory of ecosystems and identify the specific taxa driving these shifts.

## 🧠 Core Architecture

The framework evaluates ecological dynamics by deconstructing them into three analytically distinct geometric layers within the retained functional space:

1. **Positional Component ($\Delta C$):** The net directional displacement of the community centroid. Measures the displacement of the community functional centroid within the retained trait space.
2. **Dispersive Component ($\Delta FDis$):** The internal demographic reorganization. Quantifies whether biomass is concentrating into central redundant strategies or expanding towards peripheral phenotypes.
3. **Boundary Component ($\Delta FRic$):** The multidimensional Convex Hull volume. Captures changes in the outer functional boundaries of communities through Convex Hull geometry. Because this metric depends exclusively on extreme species, it should be interpreted as a topological descriptor rather than a robust abundance-weighted estimate of functional change.

What does each layer measure?

ΔC     → Where is the community moving?
ΔFDis  → How is biomass being redistributed?
ΔFRic  → Are functional boundaries expanding or collapsing?

### The Triad of Null Models
Rather than relying on a single null expectation, ascent implements a hierarchical inferential framework that sequentially isolates:
* **Structural Turnover (Incidence/Curveball):** Does the taxonomic turnover alter the topology more than expected by chance?
* **Demographic reorganization (Quantitative Filter - Demographic/SAD):** Is the internal biomass redirection deterministic?
* **Functional identity effects (Identity Filter - Trait Shuffle):** Given the observed richness and abundances, are the selected functional strategies significantly different from a random draw of the regional pool?

### Species-level Functional Leverage

The asc_transitions() module projects species-specific abundance changes onto the observed ecosystem trajectory, identifying the taxa responsible for driving functional restructuring.
This allows users to move beyond community-level metrics and directly quantify the contribution of individual species to directional ecosystem change.

### Why not just use Functional Beta Diversity?

Current frameworks quantify the magnitude of functional turnover between communities.
ascent instead decomposes restructuring into three distinct mechanisms:

| Question                                          | Classical beta diversity | ascent |
| ------------------------------------------------- | ------------------------ | ------ |
| How much changed?                                 | ✓                        | ✓      |
| In which direction did it change?                 | ✗                        | ✓      |
| Did the centroid move?                            | ✗                        | ✓      |
| Did internal functional dispersion change?        | ✗                        | ✓      |
| Did the functional boundaries expand or collapse? | ✗                        | ✓      |
| Which species drove the shift?                    | ✗                        | ✓      |



## 🚀 Installation

You can install the development version of `ascent` from GitHub with:

```r
# install.packages("devtools")
devtools::install_github("V3ndetta96/ascent")
```

## 💻 Quick Start

Here is a basic workflow evaluating the functional impact of deforestation on a bird community using mixed trait data (quantitative and binary).

```r
library(ascent)

# 1. Define Traits and Abundances
# Note: Binary traits are set as factors to use the Gower metric.
traits <- data.frame(
  Body_Mass = c(15, 20, 30, 45, 60, 90, 150, 250, 400, 600),
  Frugivore = factor(c(0, 0, 1, 1, 1, 0, 1, 1, 1, 0))
)
rownames(traits) <- paste0("Sp", 1:10)

abund_time <- rbind(
  Reference = c( 0,  0, 10, 15, 25, 20, 10, 8, 4, 2), # Old-growth forest
  Impacted  = c(40, 35, 15,  5,  0,  0,  0, 0, 0, 0)  # Deforested
)

# 2. Execute the Paired Multi-Layer Framework
res <- asc_paired(
  traits = traits, abund = abund_time, 
  sites = c("Site1", "Site1"), time = c("Reference", "Impacted"),
  ref_time = "Reference", dist_method = "gower"
)

# 3. Evaluate Hierarchical Null Models
res <- asc_null(res, n_perm = 999, seed = 123)

# 4. View Automated Ecological Diagnostic
summary(res)
```

### Visualizing Dynamic Topology and Functional Leverage

`ascent` includes built-in `ggplot2`/`patchwork` methods to visualize the PCoA trajectory, the 2D Convex Hull boundaries, and the Functional Leverage (the specific taxa pulling or releasing the centroid).

```r
plot(res, contrast = "Site1", type = "both", n_sp = 5)
```

## 🛠️ Package API Overview

| Function | Purpose |
|---|---|
| `asc_baseline()` | Absolute baseline functional topology (CWM, FDis, FRic) for isolated communities |
| `asc_paired()` | Temporal or paired experimental contrasts |
| `asc_pairwise()` | Bidirectional spatial divergence across all community pairs |
| `asc_null()` | Triad of null models (Structural, Quantitative, Identity) |
| `asc_transitions()` | Multidimensional leverage score for every species |
| `asc_entities()` | Species clustering into discrete functional entities |
| `assess_functional_space()` | Diagnostic tool to evaluate PCoA quality and dimensionality |

## 📖 Documentation & Vignettes

For a comprehensive guide, including spatial network analyses and detailed ecological interpretations, please read the official vignette:

```r
vignette("ascent_tutorial")
```

## ⚠️ Methodological Notes

### FRic (Functional Richness)

FRic is computed as the volume of the convex hull in the retained PCoA space. Key considerations:

- **Depends exclusively on extreme species.** FRic captures the outer boundary of the functional space. It is insensitive to internal reorganization of biomass. A species that increases from 1% to 90% relative abundance will not alter FRic if it is not on the hull boundary.
- **Not abundance-weighted.** Unlike FDis or the centroid, FRic is a purely incidence-based metric within the framework. It should be interpreted as a **topological descriptor** of the functional boundary, not as a robust estimate of functional change.
- **Requires N > k.** When a community has fewer species than retained PCoA axes, the convex hull is geometrically undefined and FRic is reported as `NA`.

### ΔFDis (Functional Dispersion Change)

- **Not normalized.** ΔFDis is an absolute difference in the retained PCoA space. Its magnitude depends on the trait scales, the number of retained axes, and the Gower distance matrix. Direct comparisons across studies with different species pools or trait sets should be made with caution.
- **Normalized position:** Only the positional component (rΔC%) is normalized by the regional functional diameter (Dmax_regional), enabling cross-study comparisons.

### Null Model Scope

`asc_null()` operates on the full stacked community matrix, assuming a **shared regional species pool**. When analyzing biogeographically independent sites, contrasts should be evaluated separately by calling `asc_null()` on each `ascfcd` object independently.

### Functional Leverage: Additive Property

The Functional Leverage satisfies a strict mathematical decomposition:

$$\sum_{i=1}^{S} \text{Leverage}_i = \|\Delta C\|$$

where $\|\Delta C\|$ is the absolute Euclidean distance between centroids. Each species' leverage is the product of its demographic change ($\Delta p_i$) and its projection onto the unit direction of the centroid shift:

$$\text{Leverage}_i = \Delta p_i \cdot \text{proj}(\mathbf{f}_i, \hat{v})$$

This property ensures that the sum of all individual species contributions exactly reconstructs the total centroid displacement. Species with positive leverage **drive** the shift; species with negative leverage **resist** it.

## ✒️ Citation

While the official manuscript is in preparation, if you use `ascent` in your research, please cite the package directly:

Muñoz-Li, R. R. & F. Alvarez-Denis (2026). ascent: A multi-layer framework for decomposing functional community restructuring into positional, dispersive and boundary components under hierarchical null models. https://github.com/V3ndetta96/ascent

#' Summary for Paired Functional Centroid Displacement
#'
#' @description
#' Provides a concise, multi-layer overview of the paired ASC-CFD analysis,
#' including null models and top functional drivers.
#'
#' @param object An object of class \code{ascfcd}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns a data frame with the displacement metrics.
#' @export
summary.ascfcd <- function(object, ...) {
  cat("==================================================\n")
  cat(" ASC-CFD: Multidimensional Functional Restructuring \n")
  cat("==================================================\n\n")

  cat(sprintf("Number of Contrasts: %d\n", length(object$rDelta_C)))
  cat(sprintf("Functional Dimensionality (k): %d (Explaining %.1f%% of variance)\n",
              object$k_retained, object$var_retained * 100))
  cat(sprintf("PCoA Quality: %.1f%%\n", object$quality * 100))

  df_dist <- data.frame(
    Contrast = names(object$rDelta_C),
    Layer1_rDeltaC = round(object$rDelta_C, 2),
    Layer2_DeltaFDis = round(sapply(object$site_results, function(x) x$Delta_FDis), 4),
    Layer3_DeltaFRic = round(sapply(object$site_results, function(x) x$Delta_FRic), 4)
  )

  cat("\n--- Functional Shift Overview ---\n")
  print(df_dist, row.names = FALSE)
  cat("\n")

  if (!is.null(object$null_models)) {
    cat("--- Multi-Level Null Model Evaluation ---\n")
    cat("Struct: Incidence Filter | Quant: Demographic Filter | Identity: Trait Filter\n")
    cat("Note: FRic under Quantitative filter is NA (incidence fixed -> hull invariant)\n\n")
    print(object$null_models, row.names = FALSE)
    cat("\n")
  } else {
    cat("Null models not evaluated. Run asc_null() to test for structural, quantitative, and identity shifts.\n\n")
  }

  # TOP 5 DRIVERS (Leverage Preview)
  cat("--- Top Functional Drivers (Leverage Preview) ---\n")
  trans <- asc_transitions(object)
  df_drivers <- data.frame(Contrast = character(), Top_5_Drivers = character(), stringsAsFactors = FALSE)

  for(s in names(trans)) {
    top5 <- utils::head(trans[[s]]$species_leverage, 5)
    drivers_str <- paste(sprintf("%s (%+.2f)", top5$Species, top5$Leverage), collapse = ", ")
    df_drivers <- rbind(df_drivers, data.frame(Contrast = s, Top_5_Drivers = drivers_str))
  }
  print(df_drivers, row.names = FALSE)
  cat("\n")

  invisible(df_dist)
}


#' Summary for Pairwise Functional Spatial Divergence
#'
#' @description
#' Provides a comprehensive overview of the spatial network ASC-CFD analysis.
#'
#' @param object An object of class \code{ascfcd_pw}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns the pairwise results data frame.
#' @export
summary.ascfcd_pw <- function(object, ...) {
  cat("==================================================\n")
  cat(" ASC-CFD: Pairwise Spatial Functional Network \n")
  cat("==================================================\n\n")

  n_com <- nrow(object$cwm_global)
  n_pairs <- nrow(object$pairwise_results)

  cat(sprintf("Communities Analyzed: %d | Spatial Contrasts: %d\n", n_com, n_pairs))
  cat(sprintf("Functional Dimensionality (k): %d (Explaining %.1f%% of variance)\n",
              object$k_retained, object$var_retained * 100))
  cat(sprintf("PCoA Quality: %.1f%%\n\n", object$quality * 100))

  cat("--- Global Divergence Summary ---\n")
  rdelta <- object$pairwise_results$rDelta_C_pct
  fdis_shifts <- object$pairwise_results$Delta_FDis
  fric_shifts <- object$pairwise_results$Delta_FRic

  cat(sprintf(" Mean Position Shift (rDelta_C): %6.2f%%\n", mean(rdelta, na.rm = TRUE)))
  cat(sprintf(" Mean Dispersion Shift (Delta_FDis): %6.4f\n", mean(fdis_shifts, na.rm = TRUE)))
  cat(sprintf(" Mean Volume Shift (Delta_FRic): %s\n\n",
              if(all(is.na(fric_shifts))) "NA" else sprintf("%6.4f", mean(fric_shifts, na.rm = TRUE))))

  if (!is.null(object$null_models)) {
    cat("--- Multi-Level Null Model Evaluation (Head) ---\n")
    cat("Struct: Incidence Filter | Quant: Demographic Filter | Identity: Trait Filter\n")
    cat("Note: FRic under Quantitative filter is NA (incidence fixed -> hull invariant)\n\n")
    print(utils::head(object$null_models, 10), row.names = FALSE)
    cat("\n")
  }

  # TOP 5 DRIVERS ESPACIALES (Leverage Preview)
  cat("--- Top Functional Drivers (Leverage Preview - Head) ---\n")
  trans <- asc_transitions(object)
  df_drivers <- data.frame(Contrast = character(), Top_5_Drivers = character(), stringsAsFactors = FALSE)

  for(s in names(trans)) {
    top5 <- utils::head(trans[[s]]$species_leverage, 5)
    drivers_str <- paste(sprintf("%s (%+.2f)", top5$Species, top5$Leverage), collapse = ", ")
    df_drivers <- rbind(df_drivers, data.frame(Contrast = s, Top_5_Drivers = drivers_str))
  }
  print(utils::head(df_drivers, 5), row.names = FALSE)
  cat("\n")

  invisible(object$pairwise_results)
}


#' Summary for Baseline Functional Entities
#'
#' @description
#' Provides an overview of the baseline functional topology of the evaluated entities.
#'
#' @param object An object of class \code{ascfcd_base}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns the entities results data frame.
#' @export
summary.ascfcd_base <- function(object, ...) {
  cat("==================================================\n")
  cat(" ASC-CFD: Baseline Functional Topology \n")
  cat("==================================================\n\n")

  cat(sprintf("Entities Analyzed: %d\n", nrow(object$entities_results)))
  cat(sprintf("Functional Dimensionality (k): %d (Explaining %.1f%% of variance)\n",
              object$k_retained, object$var_retained * 100))
  cat(sprintf("PCoA Quality: %.1f%%\n\n", object$quality * 100))

  cat("--- Absolute Entity Metrics ---\n")

  # Redondeo para limpieza visual en consola
  print_df <- object$entities_results
  num_cols <- sapply(print_df, is.numeric)
  print_df[, num_cols] <- round(print_df[, num_cols], 4)

  print(print_df, row.names = FALSE)
  cat("\n")

  invisible(object$entities_results)
}

#' Summary for Functional Entities Classification
#'
#' @description
#' Provides an overview of the species clustering into functional entities.
#'
#' @param object An object of class \code{ascfcd_entities}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns the classification data frame.
#' @export
summary.ascfcd_entities <- function(object, ...) {
  cat("==================================================\n")
  cat(" ASC-CFD: Functional Entities Classification \n")
  cat("==================================================\n\n")

  df <- object$entity_classification
  n_spp <- nrow(df)
  n_ent <- length(unique(df$Entity_ID))

  cat(sprintf("Total Species: %d\n", n_spp))
  cat(sprintf("Functional Entities Identified: %d\n", n_ent))
  cat(sprintf("Distance Metric: %s | Clustering Method: %s\n\n",
              tools::toTitleCase(object$metric), object$method))

  cat("--- Species Distribution per Entity ---\n")
  dist_table <- table(df$Entity_ID)
  print(dist_table)
  cat("\n")

  cat("--- Classification Preview (Head) ---\n")
  print(utils::head(df, 5), row.names = FALSE)
  cat("\n")

  invisible(df)
}

#' Summary for Functional Leverage Analysis
#'
#' @description
#' Provides an overview of the species leverage for each contrast.
#'
#' @param object An object of class \code{ascfcd_transitions}.
#' @param n_sp Integer. Number of top species to display per contrast. Default is 5.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns the object.
#' @export
summary.ascfcd_transitions <- function(object, n_sp = 5, ...) {
  cat("==================================================\n")
  cat(" ASC-CFD: Functional Leverage Analysis \n")
  cat("==================================================\n\n")

  cat(sprintf("Contrasts Analyzed: %d\n\n", length(object)))

  for (s in names(object)) {
    cat(sprintf("--- %s ---\n", s))
    df <- object[[s]]$species_leverage
    print_df <- utils::head(df, n_sp)
    # Round for display only
    num_cols <- sapply(print_df, is.numeric)
    print_df[, num_cols] <- round(print_df[, num_cols], 4)
    print(print_df, row.names = FALSE)
    cat("\n")
  }

  invisible(object)
}

# ==============================================================================
# INTERNAL UTILITY FUNCTIONS — SHARED ENGINES
# ==============================================================================
# These are hidden (.dot) functions that serve as the single source of truth
# for the core mathematical engines used across the package.

#' Build the Functional PCoA Space from a Trait Matrix
#'
#' @description
#' Internal function. Computes Gower (or other) distance, runs PCoA,
#' selects axes via variance threshold or broken stick, and reports quality.
#'
#' @param traits Data frame or matrix of functional traits.
#' @param dist_method Distance metric (default \code{"gower"}).
#' @param dim_retention Method for axis selection (\code{"variance"} or \code{"broken_stick"}).
#' @param var_tol Proportion of cumulative variance to retain.
#'
#' @return A list with: \code{F_mat}, \code{k_retained}, \code{var_retained},
#'   \code{axis_var}, \code{quality}.
#'
#' @details
#' **Negative eigenvalues:** Gower distance is not strictly Euclidean, so
#' \code{cmdscale} may produce negative eigenvalues. This function filters
#' them out and reports a quality metric (\code{quality = sum(positive_eig) /
#' sum(abs(all_eig))}). If quality falls below 0.80, a warning is issued.
#' No Cailliez or Lingoes correction is applied; the distances in the
#' retained PCoA space are an approximation of the original Gower distances.
#' For matrices with high proportions of binary/categorical traits, the
#' distortion may be substantial.
#'
#' @noRd
.build_functional_space <- function(traits, dist_method = "gower",
                                     dim_retention = "variance", var_tol = 0.80) {

  d_mat <- cluster::daisy(traits, metric = dist_method)
  pcoa_res <- stats::cmdscale(d_mat, k = nrow(traits) - 1, eig = TRUE)

  eig_raw <- pcoa_res$eig
  eig_pos <- eig_raw[eig_raw > 1e-8]
  rel_var <- eig_pos / sum(eig_pos)
  cum_var <- cumsum(rel_var)

  # PCoA quality: proportion of total absolute eigenvalue explained by positive axes

  quality <- sum(eig_pos) / sum(abs(eig_raw))
  if (quality < 0.80) {
    warning(sprintf(
      paste0("PCoA quality is low (%.1f%%). Negative eigenvalues represent ",
             "%.1f%% of total variation. Consider applying Cailliez correction ",
             "or reviewing trait data."),
      quality * 100, (1 - quality) * 100
    ))
  }

  if (dim_retention == "variance") {
    k_valid <- min(which(cum_var >= var_tol))
  } else if (dim_retention == "broken_stick") {
    n_eig <- length(eig_pos)
    bs_expected <- sapply(1:n_eig, function(k) sum(1 / (k:n_eig)) / n_eig)
    k_valid <- sum(rel_var > bs_expected)
  }
  if (k_valid < 2) k_valid <- 2

  axis_var <- rel_var[1:k_valid]
  total_var_retained <- cum_var[k_valid]
  F_mat <- pcoa_res$points[, 1:k_valid, drop = FALSE]
  colnames(F_mat) <- paste0("Axis_", 1:k_valid)

  list(
    F_mat = F_mat,
    k_retained = k_valid,
    var_retained = total_var_retained,
    axis_var = axis_var,
    quality = quality
  )
}


#' Compute Core Topology (FDis, FRic) for a Single Community
#'
#' @description
#' Internal function. Calculates abundance-weighted functional dispersion
#' (Laliberte & Legendre 2010) and convex hull volume for a single
#' community vector in the retained functional space.
#'
#' @param abund_vec Numeric vector of (relative) abundances.
#' @param F_space Matrix of species coordinates in PCoA space.
#'
#' @return A list with \code{FDis} and \code{FRic}.
#'   \code{FRic} is \code{NA_real_} when the convex hull is not computable.
#' @noRd
.calc_core_topology <- function(abund_vec, F_space) {
  idx <- abund_vec > 0
  n_pres <- sum(idx)

  if (n_pres < 2) return(list(FDis = 0, FRic = NA_real_))

  p_pres <- abund_vec[idx] / sum(abund_vec[idx])
  F_pres <- F_space[idx, , drop = FALSE]

  centroid <- colSums(p_pres * F_pres)
  dist_to_c <- sqrt(rowSums(sweep(F_pres, 2, centroid, "-")^2))
  fdis <- sum(p_pres * dist_to_c)

  fric <- NA_real_
  if (n_pres > ncol(F_space)) {
    fric <- tryCatch(
      geometry::convhulln(F_pres, options = "FA")$vol,
      error = function(e) NA_real_
    )
  }

  list(FDis = fdis, FRic = fric)
}

#' Extract Functional Transitions and Species Leverage
#'
#' @description
#' Calculates the multidimensional Functional Leverage of each species,
#' quantifying their direct contribution to the positional shift (Layer 1 - Delta C)
#' of the ecosystem.
#'
#' @param x An object of class \code{ascfcd} or \code{ascfcd_pw}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An S3 object of class \code{ascfcd_transitions}.
#'
#' @examples
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85)
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' abund <- rbind(
#'   Ref = c(0, 10, 25, 20, 5),
#'   Imp = c(30, 15,  5,  0, 0)
#' )
#'
#' res <- asc_paired(
#'   traits, abund,
#'   sites = c("S1", "S1"),
#'   time = c("R", "I"), ref_time = "R",
#'   dist_method = "euclidean"
#' )
#' drivers <- asc_transitions(res)
#' drivers$S1$species_leverage
#'
#' @export
asc_transitions <- function(x, ...) {
  if (!inherits(x, c("ascfcd", "ascfcd_pw"))) {
    stop("Input must be of class 'ascfcd' or 'ascfcd_pw'.")
  }

  F_mat <- x$F_space
  entities <- names(x$directional_vectors)
  res_list <- list()

  for (s in entities) {
    v_dir <- x$directional_vectors[[s]]
    mag_v <- sqrt(sum(v_dir^2))

    # Extraemos la demografía
    p_r <- as.numeric(x$p_ref[[s]])
    p_c <- as.numeric(x$p_comp[[s]])
    delta_p <- p_c - p_r

    if (mag_v < 1e-12) {
      # Si no hay desplazamiento, el leverage es nulo
      df_lev <- data.frame(Species = rownames(F_mat), Delta_p = delta_p, Projection = 0, Leverage = 0)
    } else {
      # Vector unitario direccional
      u_dir <- v_dir / mag_v

      # Ubicamos el centroide de origen según el tipo de objeto
      if (inherits(x, "ascfcd")) {
        c_base <- as.numeric(x$cwm_ref[[s]])
      } else {
        com_A <- strsplit(s, "_vs_")[[1]][1]
        c_base <- as.numeric(x$cwm_global[com_A, ])
      }

      # Proyección ortogonal de cada especie sobre el vector direccional
      projections <- numeric(nrow(F_mat))
      for (i in seq_len(nrow(F_mat))) {
        projections[i] <- sum((F_mat[i, ] - c_base) * u_dir)
      }

      # Cálculo del Leverage Multidimensional
      leverage <- delta_p * projections

      df_lev <- data.frame(
        Species = rownames(F_mat),
        Delta_p = delta_p,
        Projection = projections,
        Leverage = leverage
      )
    }

    # Ordenamos por impacto absoluto (los mayores drivers arriba)
    df_lev <- df_lev[order(abs(df_lev$Leverage), decreasing = TRUE), ]
    rownames(df_lev) <- NULL

    res_list[[s]] <- list(species_leverage = df_lev)
  }

  class(res_list) <- "ascfcd_transitions"
  return(res_list)
}

# ==============================================================================
# test-core.R — Unit tests for ascent core functions
# ==============================================================================
# Covers: asc_paired, asc_pairwise, asc_null, asc_transitions
# Orthogonal scenarios: pure translation, pure reorganization, pure expansion
# ==============================================================================

# --- Shared fixtures ---
traits_10sp <- data.frame(
  Body_Mass   = c(15, 20, 30, 45, 60, 90, 150, 250, 400, 600),
  Beak_Length = c(10, 12, 15, 22, 28, 35,  45,  60,  85, 120),
  Frugivore   = factor(c(0, 0, 1, 1, 1, 0,  1,  1,  1,  0)),
  Forest_Dep  = factor(c(0, 0, 0, 0, 1, 1,  1,  1,  1,  1))
)
rownames(traits_10sp) <- paste0("Sp", 1:10)


# ==============================================================================
# asc_paired
# ==============================================================================
test_that("asc_paired returns correct class and structure", {
  abund <- rbind(
    Ref = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    Imp = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")

  expect_s3_class(res, "ascfcd")
  expect_true("rDelta_C" %in% names(res))
  expect_true("quality" %in% names(res))
  expect_true(is.numeric(res$quality) && res$quality > 0)
  expect_true("S1" %in% names(res$rDelta_C))
  expect_true(res$rDelta_C["S1"] > 0)
})

test_that("asc_paired handles degenerate community (1 species)", {
  abund <- rbind(
    Ref = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1),
    Imp = c(100, 0, 0, 0, 0, 0, 0, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")

  expect_true(is.na(res$site_results$S1$Delta_FRic))
  expect_true(!is.na(res$rDelta_C["S1"]))
})

test_that("asc_paired returns zero shift for identical communities", {
  abund <- rbind(
    A = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1),
    B = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "C"), ref_time = "R")

  expect_equal(unname(res$rDelta_C["S1"]), 0, tolerance = 1e-10)
})


# ==============================================================================
# ORTHOGONAL SCENARIOS
# ==============================================================================

# --- Scenario 1: Pure Translation ---
# Same species, same richness, shifted abundances → position moves, dispersion ~stable
test_that("Pure translation: position shifts, dispersion stable", {
  abund <- rbind(
    Ref  = c(0, 0,  5, 10, 40, 30, 10,  3, 1, 1),
    Comp = c(0, 0,  1,  3, 10, 30, 40, 10, 5, 1)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "C"), ref_time = "R")

  expect_true(res$rDelta_C["S1"] > 0)
  # FRic should be NA or zero (same species present, same hull)
  # Actually both have same species so Delta_FRic should be ~0
  delta_fric <- res$site_results$S1$Delta_FRic
  if (!is.na(delta_fric)) {
    expect_equal(delta_fric, 0, tolerance = 1e-8)
  }
})

# --- Scenario 2: Pure Reorganization ---
# Same centroid, different spread → dispersion changes, position ~stable
test_that("Pure reorganization: dispersion shifts, position near-zero", {
  # Uniform → concentrated on central species
  abund <- rbind(
    Ref  = c(0, 0, 10, 10, 10, 10, 10, 10, 10, 10),
    Comp = c(0, 0,  1,  1,  1,  1, 40, 40,  1,  1)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "C"), ref_time = "R")

  # FDis should change
  expect_true(abs(res$site_results$S1$Delta_FDis) > 0)
})

# --- Scenario 3: Pure Expansion ---
# Outer species added → volume expands, centroid may shift slightly
test_that("Pure expansion: FRic changes with species loss", {
  abund <- rbind(
    Ref  = c(5, 5, 10, 15, 25, 20, 10, 8, 5, 5),
    Comp = c(0, 0,  0, 15, 25, 20, 10, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "C"), ref_time = "R")

  delta_fric <- res$site_results$S1$Delta_FRic
  # May be NA if comp has too few species for hull
  if (!is.na(delta_fric)) {
    expect_true(delta_fric != 0)
  }
})


# ==============================================================================
# asc_pairwise
# ==============================================================================
test_that("asc_pairwise returns correct class and handles multiple communities", {
  abund <- rbind(
    A = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    B = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0),
    C = c(0, 5, 10, 20, 35, 20, 10, 0, 0, 0)
  )

  res <- asc_pairwise(traits_10sp, abund)

  expect_s3_class(res, "ascfcd_pw")
  expect_equal(nrow(res$pairwise_results), 3)  # 3 choose 2
  expect_true(all(res$pairwise_results$rDelta_C_pct >= 0))
})


# ==============================================================================
# asc_null
# ==============================================================================
test_that("asc_null appends null_models with correct structure", {
  abund <- rbind(
    Ref = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    Imp = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")

  res <- asc_null(res, n_perm = 10, seed = 42)

  expect_true("null_models" %in% names(res))
  expect_s3_class(res$null_models, "data.frame")
  expect_true(all(c("Filter", "Metric", "SES", "P_value") %in% names(res$null_models)))
})

test_that("asc_null reproducibility with same seed", {
  abund <- rbind(
    Ref = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    Imp = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0)
  )

  base <- asc_paired(traits_10sp, abund,
                      sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")

  r1 <- asc_null(base, n_perm = 50, seed = 123)
  r2 <- asc_null(base, n_perm = 50, seed = 123)

  expect_identical(r1$null_models, r2$null_models)
})

test_that("asc_null Model B FRic is always NA", {
  abund <- rbind(
    Ref  = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1),
    Comp = c(1, 2, 4, 8, 10, 25, 20, 15, 10, 5)
  )

  res <- asc_null(
    asc_paired(traits_10sp, abund,
               sites = c("S", "S"), time = c("R", "C"), ref_time = "R"),
    n_perm = 10, seed = 1
  )

  fric_b <- res$null_models[res$null_models$Filter == "Quantitative" &
                             res$null_models$Metric == "Volume (Delta FRic)", ]
  expect_true(all(is.na(fric_b$SES)))
  expect_true(all(is.na(fric_b$P_value)))
})


# ==============================================================================
# asc_transitions
# ==============================================================================
test_that("asc_transitions returns class and leverage additivity", {
  abund <- rbind(
    Ref = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    Imp = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")
  trans <- asc_transitions(res)

  expect_s3_class(trans, "ascfcd_transitions")
  expect_true("S1" %in% names(trans))

  # Leverage additivity: sum(Lev) = ||Delta_C||
  lev_sum <- sum(trans$S1$species_leverage$Leverage)
  abs_dist <- res$site_results$S1$abs_dist
  expect_equal(lev_sum, abs_dist, tolerance = 1e-8)
})

test_that("asc_transitions zero displacement gives zero leverage", {
  abund <- rbind(
    A = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1),
    B = c(5, 10, 15, 20, 25, 10, 8, 4, 2, 1)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "C"), ref_time = "R")
  trans <- asc_transitions(res)

  expect_true(all(trans$S1$species_leverage$Leverage == 0))
})

test_that("Leverage = Delta_p * Projection", {
  abund <- rbind(
    Ref = c(0, 0, 10, 15, 25, 20, 10, 8, 4, 2),
    Imp = c(40, 35, 15, 5, 0, 0, 0, 0, 0, 0)
  )

  res <- asc_paired(traits_10sp, abund,
                     sites = c("S1", "S1"), time = c("R", "I"), ref_time = "R")
  trans <- asc_transitions(res)
  df <- trans$S1$species_leverage

  recomputed <- df$Delta_p * df$Projection
  expect_equal(df$Leverage, recomputed, tolerance = 1e-12)
})

# ==============================================================================
# PRINT METHODS FOR ALL S3 CLASSES
# ==============================================================================

#' @export
print.ascfcd <- function(x, ...) {
  cat("ASC-CFD Paired Analysis\n")
  cat(sprintf("  Contrasts: %d | Dimensions: %d (%.1f%% var | Quality: %.1f%%)\n",
              length(x$rDelta_C), x$k_retained, x$var_retained * 100, x$quality * 100))
  cat(sprintf("  Null models: %s\n", if(!is.null(x$null_models)) "evaluated" else "not evaluated"))
  cat("Use summary() for full results.\n")
  invisible(x)
}

#' @export
print.ascfcd_pw <- function(x, ...) {
  cat("ASC-CFD Pairwise Spatial Analysis\n")
  cat(sprintf("  Communities: %d | Contrasts: %d | Dimensions: %d (%.1f%% var | Quality: %.1f%%)\n",
              nrow(x$cwm_global), nrow(x$pairwise_results), x$k_retained,
              x$var_retained * 100, x$quality * 100))
  cat(sprintf("  Null models: %s\n", if(!is.null(x$null_models)) "evaluated" else "not evaluated"))
  cat("Use summary() for full results.\n")
  invisible(x)
}

#' @export
print.ascfcd_base <- function(x, ...) {
  cat("ASC-CFD Baseline Functional Topology\n")
  cat(sprintf("  Entities: %d | Dimensions: %d (%.1f%% var | Quality: %.1f%%)\n",
              nrow(x$entities_results), x$k_retained, x$var_retained * 100, x$quality * 100))
  cat("Use summary() for full results.\n")
  invisible(x)
}

#' @export
print.ascfcd_entities <- function(x, ...) {
  n_ent <- length(unique(x$entity_classification$Entity_ID))
  n_spp <- nrow(x$entity_classification)
  cat("ASC-CFD Functional Entities Classification\n")
  cat(sprintf("  Species: %d | Entities: %d | Metric: %s | Method: %s\n",
              n_spp, n_ent, x$metric, x$method))
  cat("Use summary() for full results.\n")
  invisible(x)
}

#' @export
print.ascfcd_transitions <- function(x, ...) {
  cat("ASC-CFD Functional Leverage Analysis\n")
  cat(sprintf("  Contrasts: %d\n", length(x)))
  cat("Use summary() for detailed results.\n")
  invisible(x)
}

#' Baseline Functional Topology of Entities
#'
#' @description
#' Calculates the absolute functional centroid (CWM), functional dispersion (FDis),
#' and convex hull volume (FRic) for each community or site independently.
#'
#' @param traits A data frame or matrix of functional traits.
#' @param abund A data frame or matrix of species abundances.
#' @param dist_method Distance metric for the trait matrix. Default is "gower".
#' @param dim_retention Character. Method for dimensionality reduction. Options are \code{"variance"} or \code{"broken_stick"}.
#' @param var_tol Numeric. Proportion of cumulative variance to retain. Default is 0.80.
#' @param na.rm Logical. Remove missing values? Default is TRUE.
#'
#' @return An S3 object of class \code{ascfcd_base}.
#'
#' @examples
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85)
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' abund <- rbind(
#'   Forest = c(0,  5, 25, 20, 10),
#'   Field  = c(30, 20, 10,  0,  0)
#' )
#'
#' base <- asc_baseline(traits, abund, dist_method = "euclidean")
#' summary(base)
#'
#' @export
asc_baseline <- function(traits, abund, dist_method = "gower",
                         dim_retention = c("variance", "broken_stick"), var_tol = 0.80, na.rm = TRUE) {

  dim_retention <- match.arg(dim_retention)

  if(nrow(traits) != ncol(abund)) stop("Number of species in traits and abundances does not match.")
  abund_rel <- as.matrix(sweep(abund, 1, rowSums(abund, na.rm = na.rm), "/"))

  fs <- .build_functional_space(traits, dist_method, dim_retention, var_tol)
  F_mat <- fs$F_mat
  k_valid <- fs$k_retained

  cwm_global <- abund_rel %*% F_mat
  entities <- rownames(abund_rel)

  res_df <- data.frame(Entity = entities)
  for(j in 1:k_valid) {
    res_df[[paste0("CWM_Axis_", j)]] <- cwm_global[, j]
  }

  fdis_vals <- numeric(length(entities))
  fric_vals <- numeric(length(entities))

  for(i in seq_along(entities)) {
    topo <- .calc_core_topology(abund_rel[i, ], F_mat)
    fdis_vals[i] <- topo$FDis
    fric_vals[i] <- topo$FRic
  }

  res_df$FDis <- fdis_vals
  res_df$FRic <- fric_vals

  output <- list(
    entities_results = res_df,
    cwm_global = cwm_global,
    F_space = F_mat,
    k_retained = k_valid,
    var_retained = fs$var_retained,
    axis_var = fs$axis_var,
    quality = fs$quality,
    original_traits = traits,
    original_abund = abund_rel
  )
  class(output) <- "ascfcd_base"
  return(output)
}

#' Paired Functional Multidimensional Restructuring
#'
#' @description
#' Calculates centroid displacement (Layer 1), dispersion shift (Layer 2 - FDis),
#' and volume shift (Layer 3 - FRic) between paired states.
#'
#' @param traits A data frame or matrix of functional traits.
#' @param abund A data frame or matrix of species abundances.
#' @param sites A vector indicating the site identifier.
#' @param time A vector indicating the temporal state.
#' @param ref_time A character string specifying the reference state.
#' @param dist_method Distance metric for the trait matrix. Default is "gower".
#' @param dim_retention Character. Method for dimensionality reduction.
#' @param var_tol Numeric. Proportion of cumulative variance to retain. Default is 0.80.
#' @param na.rm Logical. Remove missing values? Default is TRUE.
#'
#' @note
#' **Normalization:** Delta FDis and Delta FRic are absolute differences, not
#' normalized by the regional pool. Values are not directly comparable across
#' studies with different species pools or trait scales. Only rDelta_C (position)
#' is normalized by Dmax_regional.
#'
#' **FRic = NA:** When a community has fewer species than retained PCoA axes,
#' the convex hull is geometrically undefined and FRic is set to \code{NA},
#' which propagates to Delta_FRic.
#'
#' **Temporal replicates:** When multiple rows share the same site and temporal
#' state (e.g., three reference plots), their relative abundances are averaged
#' (column means) before computing the centroid. This is equivalent to treating
#' replicates as a single pooled community. Intra-state variability is not
#' propagated to downstream metrics.
#'
#' @return An S3 object of class \code{ascfcd}.
#'
#' @examples
#' # Simulate deforestation impact on a bird community
#' traits <- data.frame(
#'   Mass  = c(15, 30, 60, 150, 400),
#'   Beak  = c(10, 15, 28,  45,  85),
#'   Diet  = factor(c(0, 1, 1, 1, 0))
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' abund <- rbind(
#'   Reference = c(0, 10, 25, 20, 5),
#'   Impacted  = c(30, 15,  5,  0, 0)
#' )
#'
#' res <- asc_paired(
#'   traits, abund,
#'   sites = c("Site1", "Site1"),
#'   time = c("Reference", "Impacted"),
#'   ref_time = "Reference"
#' )
#' summary(res)
#'
#' @export
asc_paired <- function(traits, abund, sites, time, ref_time, dist_method = "gower",
                       dim_retention = c("variance", "broken_stick"), var_tol = 0.80, na.rm = TRUE) {

  dim_retention <- match.arg(dim_retention)

  if(nrow(traits) != ncol(abund)) stop("Number of species in traits and abundances does not match.")
  abund_rel <- as.matrix(sweep(abund, 1, rowSums(abund, na.rm = na.rm), "/"))

  fs <- .build_functional_space(traits, dist_method, dim_retention, var_tol)
  F_mat <- fs$F_mat

  cwm_global <- abund_rel %*% F_mat
  Dmax_regional <- max(stats::dist(F_mat))
  if(Dmax_regional == 0) Dmax_regional <- 1

  unique_sites <- unique(sites)
  res_list <- list()
  rDelta_C <- numeric(length(unique_sites))
  names(rDelta_C) <- unique_sites
  directional_vectors <- list(); p_ref_list <- list(); p_comp_list <- list()
  cwm_ref_list <- list(); cwm_comp_list <- list()

  for (s in unique_sites) {
    idx <- which(sites == s)
    abund_site <- abund_rel[idx, , drop = FALSE]
    time_site <- time[idx]

    idx_ref <- which(time_site == ref_time)
    idx_comp <- which(time_site != ref_time)

    if(length(idx_ref) == 0 || length(idx_comp) == 0) { rDelta_C[s] <- NA; next }

    p_ref_mat <- if(nrow(abund_site[idx_ref, , drop=FALSE]) > 1) t(as.matrix(colMeans(abund_site[idx_ref, , drop=FALSE]))) else abund_site[idx_ref, , drop=FALSE]
    p_comp_mat <- if(nrow(abund_site[idx_comp, , drop=FALSE]) > 1) t(as.matrix(colMeans(abund_site[idx_comp, , drop=FALSE]))) else abund_site[idx_comp, , drop=FALSE]

    p_ref_list[[s]] <- p_ref_mat; p_comp_list[[s]] <- p_comp_mat
    cwm_ref <- p_ref_mat %*% F_mat; cwm_comp <- p_comp_mat %*% F_mat
    cwm_ref_list[[s]] <- cwm_ref; cwm_comp_list[[s]] <- cwm_comp

    v_dir <- cwm_comp - cwm_ref
    directional_vectors[[s]] <- v_dir
    dist_abs <- sqrt(sum(v_dir^2))
    rDelta_C[s] <- (dist_abs / Dmax_regional) * 100

    # Guard: zero displacement
    ss_vdir <- sum(v_dir^2)
    if (ss_vdir < 1e-12) {
      asc_contrib <- rep(0, length(v_dir))
      crit_axis <- NA_integer_
    } else {
      asc_contrib <- (v_dir^2 / ss_vdir) * 100
      crit_axis <- which.max(asc_contrib)
    }

    topo_ref <- .calc_core_topology(as.numeric(p_ref_mat), F_mat)
    topo_comp <- .calc_core_topology(as.numeric(p_comp_mat), F_mat)

    res_list[[s]] <- list(
      abs_dist = dist_abs,
      asc = asc_contrib,
      critical_axis = crit_axis,
      Delta_FDis = topo_comp$FDis - topo_ref$FDis,
      Delta_FRic = topo_comp$FRic - topo_ref$FRic
    )
  }

  valid_sites <- !is.na(rDelta_C)
  output <- list(
    rDelta_C = rDelta_C[valid_sites],
    Dmax_regional = Dmax_regional,
    directional_vectors = directional_vectors,
    p_ref = p_ref_list, p_comp = p_comp_list,
    cwm_ref = cwm_ref_list, cwm_comp = cwm_comp_list,
    site_results = res_list[valid_sites],
    F_space = F_mat, k_retained = fs$k_retained,
    var_retained = fs$var_retained, axis_var = fs$axis_var,
    quality = fs$quality,
    original_traits = traits, original_abund = abund_rel
  )
  class(output) <- "ascfcd"
  return(output)
}

# ==============================================================================
# DECLARACIÓN DE VARIABLES GLOBALES
# ==============================================================================
# Esto evita las advertencias de 'R CMD check' por el uso de evaluación
# no estándar en ggplot2 (funciones aes()).

if(getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    # Variables usadas en plot.ascfcd y plot.ascfcd_pw
    "PC1",
    "PC2",
    "State",
    "Community",
    "Leverage",
    "Species",

    # Variables usadas en los gráficos de transiciones / Delta CWM
    "Trait",
    "Delta_CWM_Std",
    "Abs_Shift"
  ))
}

#' Plot Multidimensional Functional Restructuring
#'
#' @description
#' Visualizes the functional trajectory (Centroid shift) and volume shift (Convex Hull)
#' of a specific site/contrast in the PCoA space, alongside the species leverage.
#'
#' @param x An object of class \code{ascfcd}.
#' @param contrast Character. The exact name of the site/contrast to plot.
#' @param type Character. What to plot? \code{"pcoa"} (Trajectory & Hull), \code{"leverage"} (Drivers), or \code{"both"}. Default is \code{"both"}.
#' @param n_sp Integer. Number of top species to display in the leverage plot. Default is 10.
#' @param ... Additional graphical arguments.
#'
#' @import ggplot2
#' @return A ggplot object (or patchwork object if \code{type = "both"}).
#' @export
plot.ascfcd <- function(x, contrast, type = c("both", "pcoa", "leverage"), n_sp = 10, ...) {

  type <- match.arg(type)
  if(!(contrast %in% names(x$rDelta_C))) {
    stop(sprintf("Contrast '%s' not found. Available: %s", contrast, paste(names(x$rDelta_C), collapse = ", ")))
  }

  # ============================================================================
  # 1. PCOA TRAJECTORY & HULL PLOT
  # ============================================================================
  F_mat <- x$F_space
  c_ref <- x$cwm_ref[[contrast]]
  c_comp <- x$cwm_comp[[contrast]]

  # Extracción de abundancias para este contraste
  p_ref <- as.numeric(x$p_ref[[contrast]])
  p_comp <- as.numeric(x$p_comp[[contrast]])
  sp_names <- rownames(F_mat)

  # Coordenadas de especies (Fondo)
  df_spp <- as.data.frame(F_mat[, 1:2, drop = FALSE])
  colnames(df_spp) <- c("PC1", "PC2")

  # Coordenadas de Centroides
  df_cent <- data.frame(
    PC1 = c(c_ref[1], c_comp[1]),
    PC2 = c(c_ref[2], c_comp[2]),
    State = factor(c("Reference", "Comparison"), levels = c("Reference", "Comparison"))
  )

  # Cálculo de polígonos Convex Hull (Capa 3)
  calc_hull_2d <- function(abund_vec, state_name) {
    idx <- abund_vec > 0
    if(sum(idx) < 3) return(data.frame(PC1=numeric(0), PC2=numeric(0), State=character(0)))
    coords <- df_spp[idx, ]
    hull_idx <- grDevices::chull(coords$PC1, coords$PC2)
    poly_df <- coords[hull_idx, ]
    poly_df$State <- state_name
    return(poly_df)
  }

  df_hulls <- rbind(
    calc_hull_2d(p_ref, "Reference"),
    calc_hull_2d(p_comp, "Comparison")
  )
  if(nrow(df_hulls) > 0) {
    df_hulls$State <- factor(df_hulls$State, levels = c("Reference", "Comparison"))
  }

  # Textos de varianza y diagnóstico
  var_p1 <- x$axis_var[1] * 100
  var_p2 <- x$axis_var[2] * 100

  res <- x$site_results[[contrast]]
  fric_text <- if(is.na(res$Delta_FRic)) "NA" else sprintf("%.4f", res$Delta_FRic)
  sub_text <- sprintf("rDelta C: %.1f%%\nDelta FDis: %.4f | Delta FRic: %s",
                      x$rDelta_C[contrast], res$Delta_FDis, fric_text)

  p_pca <- ggplot() +
    geom_point(data = df_spp, aes(x = PC1, y = PC2), color = "grey85", size = 1.5, alpha = 0.8)

  # Añadir polígonos si existen
  if(nrow(df_hulls) > 0) {
    p_pca <- p_pca + geom_polygon(data = df_hulls, aes(x = PC1, y = PC2, fill = State, color = State), alpha = 0.15, linewidth = 0.5)
  }

  p_pca <- p_pca +
    geom_segment(data = df_cent, aes(x = PC1[1], y = PC2[1], xend = PC1[2], yend = PC2[2]),
                 arrow = arrow(length = unit(0.2, "cm"), type = "closed"), color = "black", linewidth = 0.8) +
    geom_point(data = df_cent, aes(x = PC1, y = PC2, color = State), size = 4) +
    scale_color_manual(values = c("Reference" = "#4575b4", "Comparison" = "#d73027")) +
    scale_fill_manual(values = c("Reference" = "#4575b4", "Comparison" = "#d73027")) +
    theme_minimal(base_size = 13) +
    labs(title = paste("Topology:", contrast), subtitle = sub_text,
         x = sprintf("PCoA 1 (%.1f%%)", var_p1), y = sprintf("PCoA 2 (%.1f%%)", var_p2)) +
    theme(legend.position = "bottom",
          legend.title = element_blank(),
          plot.subtitle = element_text(size = 10, lineheight = 1.2, color = "grey30"))

  if (type == "pcoa") return(p_pca)

  # ============================================================================
  # 2. FUNCTIONAL LEVERAGE PLOT
  # ============================================================================
  trans <- asc_transitions(x)
  p_lev <- .plot_leverage(trans, contrast, n_sp)

  if (type == "leverage") return(p_lev)

  if (!requireNamespace("patchwork", quietly = TRUE)) {
    warning("Package 'patchwork' is required to plot type = 'both'. Returning PCoA only.")
    return(p_pca)
  }
  return(p_pca | p_lev)
}

#' Plot Pairwise Functional Spatial Divergence
#'
#' @description
#' Visualizes spatial functional restructuring (Centroid & Hull) between two communities.
#'
#' @param x An object of class \code{ascfcd_pw}.
#' @param contrast Character. The contrast to plot (e.g., "SiteA_vs_SiteB").
#' @param type Character. What to plot? \code{"pcoa"}, \code{"leverage"}, or \code{"both"}.
#' @param n_sp Integer. Number of top species to display. Default is 10.
#' @param ... Additional graphical arguments.
#'
#' @import ggplot2
#' @return A ggplot object.
#' @export
plot.ascfcd_pw <- function(x, contrast, type = c("both", "pcoa", "leverage"), n_sp = 10, ...) {

  type <- match.arg(type)
  if(!(contrast %in% names(x$directional_vectors))) {
    stop(sprintf("Contrast '%s' not found. Available: %s", contrast, paste(utils::head(names(x$directional_vectors)), collapse = ", ")))
  }

  F_mat <- x$F_space
  coms <- unlist(strsplit(contrast, "_vs_"))
  com_A <- coms[1]; com_B <- coms[2]
  c_A <- x$cwm_global[com_A, ]; c_B <- x$cwm_global[com_B, ]

  df_spp <- as.data.frame(F_mat[, 1:2, drop = FALSE])
  colnames(df_spp) <- c("PC1", "PC2")

  df_cent <- data.frame(
    PC1 = c(c_A[1], c_B[1]), PC2 = c(c_A[2], c_B[2]),
    Community = factor(c(com_A, com_B), levels = c(com_A, com_B))
  )

  calc_hull_2d <- function(abund_vec, state_name) {
    idx <- abund_vec > 0
    if(sum(idx) < 3) return(data.frame(PC1=numeric(0), PC2=numeric(0), Community=character(0)))
    coords <- df_spp[idx, ]
    hull_idx <- grDevices::chull(coords$PC1, coords$PC2)
    poly_df <- coords[hull_idx, ]
    poly_df$Community <- state_name
    return(poly_df)
  }

  df_hulls <- rbind(
    calc_hull_2d(as.numeric(x$p_ref[[contrast]]), com_A),
    calc_hull_2d(as.numeric(x$p_comp[[contrast]]), com_B)
  )
  if(nrow(df_hulls) > 0) df_hulls$Community <- factor(df_hulls$Community, levels = c(com_A, com_B))

  res <- x$pairwise_results[x$pairwise_results$Community_A == com_A & x$pairwise_results$Community_B == com_B, ]
  fric_text <- if(is.na(res$Delta_FRic)) "NA" else sprintf("%.4f", res$Delta_FRic)
  sub_text <- sprintf("rDelta C: %.1f%%\nDelta FDis: %.4f | Delta FRic: %s",
                      res$rDelta_C_pct, res$Delta_FDis, fric_text)

  p_pca <- ggplot() +
    geom_point(data = df_spp, aes(x = PC1, y = PC2), color = "grey85", size = 1.5, alpha = 0.8)

  if(nrow(df_hulls) > 0) {
    p_pca <- p_pca + geom_polygon(data = df_hulls, aes(x = PC1, y = PC2, fill = Community, color = Community), alpha = 0.15, linewidth = 0.5)
  }

  p_pca <- p_pca +
    geom_segment(data = df_cent, aes(x = PC1[1], y = PC2[1], xend = PC1[2], yend = PC2[2]),
                 arrow = arrow(length = unit(0.2, "cm"), type = "closed"), color = "black", linewidth = 0.8) +
    geom_point(data = df_cent, aes(x = PC1, y = PC2, color = Community), size = 4) +
    scale_color_manual(values = stats::setNames(c("#4575b4", "#d73027"), c(com_A, com_B))) +
    scale_fill_manual(values = stats::setNames(c("#4575b4", "#d73027"), c(com_A, com_B))) +
    theme_minimal(base_size = 13) +
    labs(title = paste("Spatial Topology:", contrast), subtitle = sub_text,
         x = sprintf("PCoA 1 (%.1f%%)", x$axis_var[1]*100), y = sprintf("PCoA 2 (%.1f%%)", x$axis_var[2]*100)) +
    theme(legend.position = "bottom",
          legend.title = element_blank(),
          plot.subtitle = element_text(size = 10, lineheight = 1.2, color = "grey30"))

  if (type == "pcoa") return(p_pca)

  # Leverage via asc_transitions (DRY)
  trans <- asc_transitions(x)
  p_lev <- .plot_leverage(trans, contrast, n_sp)

  if (type == "leverage") return(p_lev)
  if (!requireNamespace("patchwork", quietly = TRUE)) return(p_pca)
  return(p_pca | p_lev)
}

#' Plot Functional Entities Dendrogram
#'
#' @description
#' Plots the hierarchical clustering dendrogram of the functional entities.
#'
#' @param x An object of class \code{ascfcd_entities}.
#' @param ... Additional graphical arguments passed to \code{plot.hclust}.
#'
#' @return No return value, called for side effects (plots a dendrogram).
#'
#' @export
plot.ascfcd_entities <- function(x, ...) {
  plot(x$hclust_obj, main = "Functional Entities Dendrogram",
       xlab = "Species", ylab = paste(tools::toTitleCase(x$metric), "Distance"),
       sub = paste("Method:", x$method), cex = 0.8, ...)

  # Si el usuario definió un número específico de clusters, dibujamos los rectángulos
  n_clusters <- length(unique(x$entity_classification$Entity_ID))
  if(n_clusters > 1) {
    stats::rect.hclust(x$hclust_obj, k = n_clusters, border = "red")
  }
}

#' Plot Functional Leverage
#'
#' @description
#' Visualizes the top species driving functional centroid displacement.
#'
#' @param x An object of class \code{ascfcd_transitions}.
#' @param contrast Character. The contrast to plot.
#' @param n_sp Integer. Number of top species to display. Default is 10.
#' @param ... Additional graphical arguments.
#'
#' @import ggplot2
#' @return A ggplot object.
#' @export
plot.ascfcd_transitions <- function(x, contrast, n_sp = 10, ...) {
  if(!(contrast %in% names(x))) {
    stop(sprintf("Contrast '%s' not found. Available: %s", contrast, paste(names(x), collapse = ", ")))
  }
  .plot_leverage(x, contrast, n_sp)
}


# ==============================================================================
# INTERNAL LEVERAGE PLOT ENGINE
# ==============================================================================
#' @noRd
.plot_leverage <- function(trans_obj, contrast, n_sp = 10) {
  df_lev <- trans_obj[[contrast]]$species_leverage
  df_lev$Abs_Lev <- abs(df_lev$Leverage)
  df_lev <- df_lev[order(df_lev$Abs_Lev, decreasing = TRUE), ]
  df_lev <- utils::head(df_lev, n_sp)
  df_lev <- df_lev[order(df_lev$Leverage), ]
  df_lev$Species <- factor(df_lev$Species, levels = df_lev$Species)

  ggplot(df_lev, aes(x = Leverage, y = Species)) +
    geom_segment(aes(x = 0, xend = Leverage, y = Species, yend = Species), color = "grey60", linewidth = 1) +
    geom_point(aes(color = Leverage > 0), size = 4) +
    scale_color_manual(values = c("TRUE" = "#d73027", "FALSE" = "#4575b4")) +
    geom_vline(xintercept = 0, color = "grey50", linewidth = 0.8) +
    theme_minimal(base_size = 13) +
    labs(title = "Functional Leverage", subtitle = "Top Drivers of Centroid Shift",
         x = "Leverage Score", y = "") +
    theme(legend.position = "none", panel.grid.minor = element_blank())
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE,
  fig.width = 10,
  fig.height = 5.5
)

## ----data_simulation----------------------------------------------------------
library(ascent)

# 1. Functional Trait Space
set.seed(42)
aves_traits <- data.frame(
  Body_Mass   = c(15, 20, 30, 45, 60, 90, 150, 250, 400, 600),
  Beak_Length = c(10, 12, 15, 22, 28, 35,  45,  60,  85, 120),
  Frugivore   = factor(c(0, 0, 1, 1, 1, 0,  1,  1,  1,  0)),
  Forest_Dep  = factor(c(0, 0, 0, 0, 1, 1,  1,  1,  1,  1))
)
rownames(aves_traits) <- paste0("Av_sp", 1:10)

# 2a. Temporal Matrix (Deforestation)
abund_time <- rbind(
  Site1_Ref = c( 0,  0, 10, 15, 25, 20, 10, 8, 4, 2), # Mature forest
  Site1_Imp = c(40, 35, 15,  5,  0,  0,  0, 0, 0, 0)  # Logged area
)

# 2b. Spatial Matrix (Landscape Gradient)
abund_space <- rbind(
  Primary   = c( 0,  0,  5, 10, 20, 25, 15, 10, 5, 2),
  Secondary = c( 0,  5, 10, 20, 35, 20, 10,  0, 0, 0),
  Agri      = c( 0,  0,  0,  5, 45, 45,  5,  0, 0, 0)
)

## ----temporal_shift-----------------------------------------------------------
res_paired <- asc_paired(
  traits = aves_traits, abund = abund_time, 
  sites = c("Site1", "Site1"), time = c("Reference", "Impacted"),
  ref_time = "Reference", dist_method = "gower"
)
res_paired <- asc_null(res_paired, n_perm = 99, seed = 123)
summary(res_paired)

## ----leverage-----------------------------------------------------------------
# Extract the specific topological drivers
drivers_time <- asc_transitions(res_paired)
head(drivers_time$Site1$species_leverage, 4)

# Plot the PCoA trajectory and the leverage divergence
plot(res_paired, contrast = "Site1", type = "both", n_sp = 5)

## ----spatial_network----------------------------------------------------------
res_pw <- asc_pairwise(traits = aves_traits, abund = abund_space, dist_method = "gower")
res_pw <- asc_null(res_pw, n_perm = 99, seed = 42)
summary(res_pw)

# Visualize the functional gap between Primary and Secondary Forest
plot(res_pw, contrast = "Primary_vs_Secondary", type = "both", n_sp = 5)

# Visualize the severe functional gap between Secondary Forest and Agriculture
plot(res_pw, contrast = "Secondary_vs_Agri", type = "both", n_sp = 5)

## ----baseline-----------------------------------------------------------------
base_topo <- asc_baseline(traits = aves_traits, abund = abund_space, dist_method = "gower")
summary(base_topo)

## ----entities, fig.width=7, fig.height=5--------------------------------------
# Cluster the species into 3 functional entities using Gower distance
func_ent <- asc_entities(traits = aves_traits, dist_method = "gower", k = 3)
summary(func_ent)

# Visualize the functional dendrogram
plot(func_ent)

#' Assess Functional Space Quality
#'
#' @description
#' Diagnostic tool to evaluate the quality of the PCoA-based functional space
#' constructed from a trait matrix. Intended for use before running core
#' analyses (\code{asc_paired}, \code{asc_pairwise}, etc.) to identify
#' potential distortion from negative eigenvalues or excessive dimensionality
#' reduction.
#'
#' @param traits A data frame or matrix of functional traits.
#' @param dist_method Distance metric for the trait matrix. Default is \code{"gower"}.
#' @param dim_retention Character. Method for dimensionality reduction.
#'   Options are \code{"variance"} or \code{"broken_stick"}. Default is \code{"variance"}.
#' @param var_tol Numeric. Proportion of cumulative variance to retain (used
#'   when \code{dim_retention = "variance"}). Default is 0.80.
#'
#' @return A list of class \code{ascfcd_space} with:
#' \describe{
#'   \item{quality}{Numeric. PCoA quality index (\code{sum(positive_eig) / sum(abs(all_eig))}). Values above 0.80 are generally acceptable.}
#'   \item{n_species}{Integer. Number of species in the trait matrix.}
#'   \item{n_traits}{Integer. Number of traits.}
#'   \item{k_retained}{Integer. Number of PCoA axes retained.}
#'   \item{var_retained}{Numeric. Cumulative variance explained by retained axes.}
#'   \item{axis_var}{Numeric vector. Relative variance per retained axis.}
#'   \item{n_neg_eigenvalues}{Integer. Count of negative eigenvalues.}
#'   \item{neg_eigenvalue_pct}{Numeric. Percentage of total absolute eigenvalue represented by negative eigenvalues.}
#'   \item{eigenvalues}{Numeric vector. All raw eigenvalues from PCoA.}
#' }
#'
#' @details
#' The Gower distance is not strictly Euclidean, which means PCoA may produce
#' negative eigenvalues. This function reports the full eigenvalue spectrum
#' and computes a quality index so users can decide whether the functional
#' space is an adequate representation of the original distance matrix.
#'
#' A quality below 0.80 indicates that negative eigenvalues represent more
#' than 20\% of the total variation. In such cases, consider:
#' \itemize{
#'   \item Reviewing the trait data for problematic variables.
#'   \item Applying a Cailliez or Lingoes correction to the distance matrix before ordination.
#'   \item Using a different distance metric (e.g., \code{"euclidean"} for purely continuous traits).
#' }
#'
#' @examples
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85),
#'   Diet = factor(c(0, 1, 1, 1, 0))
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' diag <- assess_functional_space(traits)
#' diag
#'
#' @export
assess_functional_space <- function(traits, dist_method = "gower",
                                     dim_retention = c("variance", "broken_stick"),
                                     var_tol = 0.80) {

  dim_retention <- match.arg(dim_retention)

  # Compute distance and PCoA
  d_mat <- cluster::daisy(traits, metric = dist_method)
  pcoa_res <- stats::cmdscale(d_mat, k = nrow(traits) - 1, eig = TRUE)

  eig_raw <- pcoa_res$eig
  eig_pos <- eig_raw[eig_raw > 1e-8]
  n_neg <- sum(eig_raw < -1e-8)
  neg_pct <- if (sum(abs(eig_raw)) > 0) {
    sum(abs(eig_raw[eig_raw < -1e-8])) / sum(abs(eig_raw)) * 100
  } else {
    0
  }

  quality <- sum(eig_pos) / sum(abs(eig_raw))

  # Axis retention (same logic as .build_functional_space)
  rel_var <- eig_pos / sum(eig_pos)
  cum_var <- cumsum(rel_var)

  if (dim_retention == "variance") {
    k_valid <- min(which(cum_var >= var_tol))
  } else if (dim_retention == "broken_stick") {
    n_eig <- length(eig_pos)
    bs_expected <- sapply(1:n_eig, function(k) sum(1 / (k:n_eig)) / n_eig)
    k_valid <- sum(rel_var > bs_expected)
  }
  if (k_valid < 2) k_valid <- 2

  output <- list(
    quality            = quality,
    n_species          = nrow(traits),
    n_traits           = ncol(traits),
    k_retained         = k_valid,
    var_retained       = cum_var[k_valid],
    axis_var           = rel_var[1:k_valid],
    n_neg_eigenvalues  = n_neg,
    neg_eigenvalue_pct = neg_pct,
    eigenvalues        = eig_raw
  )
  class(output) <- "ascfcd_space"
  return(output)
}


#' @export
print.ascfcd_space <- function(x, ...) {
  cat("==================================================\n")
  cat(" Functional Space Diagnostic (ascent)\n")
  cat("==================================================\n\n")
  cat(sprintf("  Species:           %d\n", x$n_species))
  cat(sprintf("  Traits:            %d\n", x$n_traits))
  cat(sprintf("  PCoA Quality:      %.1f%%\n", x$quality * 100))
  cat(sprintf("  Axes Retained:     %d (%.1f%% variance)\n",
              x$k_retained, x$var_retained * 100))
  cat(sprintf("  Negative Eigenvalues: %d (%.1f%% of total)\n",
              x$n_neg_eigenvalues, x$neg_eigenvalue_pct))

  if (x$quality < 0.80) {
    cat("\n  [!] WARNING: Quality < 80%. Consider Cailliez correction\n")
    cat("      or reviewing trait data.\n")
  } else {
    cat("\n  Quality is adequate for analysis.\n")
  }
  invisible(x)
}


#' @export
summary.ascfcd_space <- function(object, ...) {
  print.ascfcd_space(object, ...)
  cat("\n--- Per-Axis Variance ---\n")
  axis_df <- data.frame(
    Axis     = paste0("Axis_", seq_along(object$axis_var)),
    Variance = round(object$axis_var * 100, 2),
    Cumul    = round(cumsum(object$axis_var) / sum(object$axis_var) * object$var_retained * 100, 2)
  )
  print(axis_df, row.names = FALSE)
  cat("\n--- Full Eigenvalue Spectrum ---\n")
  eig <- object$eigenvalues
  cat(sprintf("  Positive: %d | Negative: %d | Total: %d\n",
              sum(eig > 1e-8), sum(eig < -1e-8), length(eig)))
  invisible(object)
}

# ==============================================================================
# test-assess.R — Unit tests for assess_functional_space
# ==============================================================================

test_that("assess_functional_space returns correct structure and values", {
  traits <- data.frame(
    Mass = c(15, 30, 60, 150, 400),
    Beak = c(10, 15, 28,  45,  85)
  )
  rownames(traits) <- paste0("Sp", 1:5)

  diag <- assess_functional_space(traits, dist_method = "euclidean")

  expect_s3_class(diag, "ascfcd_space")
  expect_equal(diag$n_species, 5)
  expect_equal(diag$n_traits, 2)
  expect_true(diag$quality > 0)
  expect_true(is.numeric(diag$eigenvalues))
})

#' Pairwise Functional Spatial Divergence
#'
#' @description
#' Calculates spatial functional divergence across three layers: position (Delta C), dispersion (FDis), and volume (FRic).
#'
#' @param traits A data frame or matrix of functional traits.
#' @param abund A data frame or matrix of species abundances.
#' @param dist_method Distance metric for the trait matrix. Default is "gower".
#' @param dim_retention Character. Method for dimensionality reduction.
#' @param var_tol Numeric. Proportion of cumulative variance to retain. Default is 0.80.
#' @param na.rm Logical. Remove missing values? Default is TRUE.
#'
#' @note
#' **Leverage direction:** In pairwise mode, leverage is computed from
#' Community_A toward Community_B. Reversing the pair order reverses the
#' leverage signs. The direction is determined by alphabetical ordering of
#' community names.
#'
#' **Normalization:** Delta FDis and Delta FRic are absolute differences (see
#' \code{\link{asc_paired}} for details).
#'
#' @return An S3 object of class \code{ascfcd_pw}.
#'
#' @examples
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85)
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' abund <- rbind(
#'   Forest  = c(0,  5, 25, 20, 10),
#'   Field   = c(30, 20, 10,  0,  0),
#'   Wetland = c(5, 10, 15, 15,  5)
#' )
#'
#' res_pw <- asc_pairwise(traits, abund, dist_method = "euclidean")
#' summary(res_pw)
#'
#' @export
asc_pairwise <- function(traits, abund, dist_method = "gower",
                         dim_retention = c("variance", "broken_stick"), var_tol = 0.80, na.rm = TRUE) {

  dim_retention <- match.arg(dim_retention)

  if(nrow(traits) != ncol(abund)) stop("Number of species in traits and abundances does not match.")
  abund_rel <- as.matrix(sweep(abund, 1, rowSums(abund, na.rm = na.rm), "/"))

  fs <- .build_functional_space(traits, dist_method, dim_retention, var_tol)
  F_mat <- fs$F_mat

  cwm_global <- abund_rel %*% F_mat
  Dmax_regional <- max(stats::dist(F_mat))
  if(Dmax_regional == 0) Dmax_regional <- 1

  communities <- rownames(abund_rel)
  combinations <- utils::combn(communities, 2, simplify = FALSE)

  results <- data.frame()
  directional_vectors <- list(); p_ref_list <- list(); p_comp_list <- list()

  for(i in seq_along(combinations)) {
    com_A <- combinations[[i]][1]
    com_B <- combinations[[i]][2]
    pair_name <- paste(com_A, com_B, sep = "_vs_")

    p_ref_mat <- abund_rel[com_A, , drop = FALSE]
    p_comp_mat <- abund_rel[com_B, , drop = FALSE]
    p_ref_list[[pair_name]] <- p_ref_mat; p_comp_list[[pair_name]] <- p_comp_mat

    cwm_A <- cwm_global[com_A, , drop = FALSE]
    cwm_B <- cwm_global[com_B, , drop = FALSE]

    v_dir <- cwm_B - cwm_A
    directional_vectors[[pair_name]] <- v_dir
    dist_abs <- sqrt(sum(v_dir^2))
    r_delta <- (dist_abs / Dmax_regional) * 100

    topo_A <- .calc_core_topology(as.numeric(p_ref_mat), F_mat)
    topo_B <- .calc_core_topology(as.numeric(p_comp_mat), F_mat)

    res_row <- data.frame(
      Community_A = com_A, Community_B = com_B,
      Delta_C_abs = dist_abs, rDelta_C_pct = r_delta,
      Delta_FDis = topo_B$FDis - topo_A$FDis,
      Delta_FRic = topo_B$FRic - topo_A$FRic
    )
    results <- rbind(results, res_row)
  }

  output <- list(
    pairwise_results = results, Dmax_regional = Dmax_regional,
    directional_vectors = directional_vectors,
    p_ref = p_ref_list, p_comp = p_comp_list,
    cwm_global = cwm_global, F_space = F_mat, k_retained = fs$k_retained,
    var_retained = fs$var_retained, axis_var = fs$axis_var,
    quality = fs$quality,
    original_traits = traits, original_abund = abund_rel
  )
  class(output) <- "ascfcd_pw"
  return(output)
}

#' Multi-Level Null Models for Multidimensional Functional Shifts
#'
#' @description
#' Evaluates the statistical significance of functional shifts using three null models:
#' Structural (Curveball), Quantitative (SAD Reshuffle), and Identity (Trait Shuffle).
#'
#' @param x An object of class \code{ascfcd} or \code{ascfcd_pw}.
#' @param n_perm Integer. Number of permutations. Default is 999.
#' @param seed Integer. Random seed for reproducibility.
#'
#' @note
#' **Model A (Structural):** After curveball permutation, all species present
#' receive uniform relative abundance (1/S_local). This means the null distribution
#' for FDis conflates the effect of taxonomic identity with the assumption of
#' equitability. The SES tests whether the observed shift is extreme given
#' random species composition, *not* given random composition with the observed SAD.
#'
#' **Model B (Quantitative):** Because incidence is held fixed, the convex hull
#' is invariant across permutations. Delta FRic under Model B is reported as
#' \code{NA} (not applicable), not zero.
#'
#' **Model C (Identity):** Statistical power is limited when the regional species
#' pool is small (< 15 species). With few species, the number of unique trait
#' permutations is small, reducing the resolution of the null distribution.
#' Consider increasing \code{n_perm} and interpreting marginal p-values
#' (0.05 < p < 0.10) with caution.
#'
#' **Scope:** When multiple contrasts are evaluated simultaneously, the curveball
#' operates on the full stacked matrix, assuming a shared regional species pool.
#' For biogeographically independent sites, run \code{asc_null()} on each
#' contrast separately.
#'
#' @importFrom vegan nullmodel
#' @importFrom stats sd simulate
#' @return The original object with an appended \code{null_models} data frame.
#'
#' @examples
#' \donttest{
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85),
#'   Diet = factor(c(0, 1, 1, 1, 0))
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' abund <- rbind(
#'   Reference = c(0, 10, 25, 20, 5),
#'   Impacted  = c(30, 15,  5,  0, 0)
#' )
#'
#' res <- asc_paired(
#'   traits, abund,
#'   sites = c("S1", "S1"),
#'   time = c("Reference", "Impacted"),
#'   ref_time = "Reference"
#' )
#' res <- asc_null(res, n_perm = 99, seed = 42)
#' res$null_models
#' }
#'
#' @export
asc_null <- function(x, n_perm = 999, seed = NULL) {

  if(!inherits(x, c("ascfcd", "ascfcd_pw"))) stop("Input object must be of class 'ascfcd' or 'ascfcd_pw'.")

  # Set seed for reproducibility if provided
  if(!is.null(seed)) set.seed(seed)

  F_mat <- x$F_space
  entities <- names(x$directional_vectors)

  # Build M matrix with explicit entity-to-row mapping
  M_list <- list()
  entity_row_ref <- integer(length(entities))
  entity_row_comp <- integer(length(entities))
  names(entity_row_ref) <- names(entity_row_comp) <- entities

  for(s in entities) {
    M_list[[paste0(s, "_ref")]] <- x$p_ref[[s]]
    M_list[[paste0(s, "_comp")]] <- x$p_comp[[s]]
  }
  M <- do.call(rbind, M_list)

  # Map each entity to its ref/comp row indices in M (explicit, not positional)
  for (j in seq_along(entities)) {
    entity_row_ref[entities[j]]  <- 2L * j - 1L
    entity_row_comp[entities[j]] <- 2L * j
  }

  # Validation: row count must equal 2 * number of entities
  stopifnot(nrow(M) == 2L * length(entities))

  M_bin <- ifelse(M > 0, 1, 0)
  nm_struct <- vegan::nullmodel(M_bin, method = "curveball")
  sims_struct <- stats::simulate(nm_struct, nsim = n_perm)

  shuffle_nonzero <- function(vec) {
    idx <- vec > 0
    if(sum(idx) > 1) vec[idx] <- sample(vec[idx])
    return(vec)
  }

  # Containers for Model A (Structural) and Model C (Identity)
  null_pos_A <- matrix(NA, n_perm, length(entities)); colnames(null_pos_A) <- entities
  null_pos_B <- matrix(NA, n_perm, length(entities)); colnames(null_pos_B) <- entities
  null_pos_C <- matrix(NA, n_perm, length(entities)); colnames(null_pos_C) <- entities

  null_fdis_A <- matrix(NA, n_perm, length(entities)); colnames(null_fdis_A) <- entities
  null_fdis_B <- matrix(NA, n_perm, length(entities)); colnames(null_fdis_B) <- entities
  null_fdis_C <- matrix(NA, n_perm, length(entities)); colnames(null_fdis_C) <- entities

  null_fric_A <- matrix(NA, n_perm, length(entities)); colnames(null_fric_A) <- entities
  # null_fric_B intentionally omitted: incidence is fixed under Model B,
  # so the convex hull is invariant. FRic has no null distribution.
  null_fric_C <- matrix(NA, n_perm, length(entities)); colnames(null_fric_C) <- entities

  for (i in 1:n_perm) {
    # Simulated matrices (A and B)
    mat_A_bin <- sims_struct[, , i]
    mat_A_rel <- sweep(mat_A_bin, 1, rowSums(mat_A_bin), "/"); mat_A_rel[is.na(mat_A_rel)] <- 0
    # M already contains relative abundances; shuffle preserves row sums.
    # Re-normalization is defensive (guards against floating-point drift).
    mat_B <- t(apply(M, 1, shuffle_nonzero))
    mat_B_rel <- mat_B  # already relative; no sweep needed

    # Model C: Shuffled Trait Space
    F_mat_shuf <- F_mat[sample(nrow(F_mat)), , drop = FALSE]

    row_idx <- 1
    for(s in entities) {
      r_idx <- entity_row_ref[s]
      c_idx <- entity_row_comp[s]

      # --- MODEL A: STRUCTURAL (Curveball) ---
      c_r_A <- mat_A_rel[r_idx, , drop=FALSE] %*% F_mat
      c_c_A <- mat_A_rel[c_idx, , drop=FALSE] %*% F_mat
      null_pos_A[i, s] <- sqrt(sum((c_c_A - c_r_A)^2))
      topo_r_A <- .calc_core_topology(mat_A_rel[r_idx, ], F_mat)
      topo_c_A <- .calc_core_topology(mat_A_rel[c_idx, ], F_mat)
      null_fdis_A[i, s] <- topo_c_A$FDis - topo_r_A$FDis
      null_fric_A[i, s] <- topo_c_A$FRic - topo_r_A$FRic

      # --- MODEL B: QUANTITATIVE (SAD Reshuffle) ---
      c_r_B <- mat_B_rel[r_idx, , drop=FALSE] %*% F_mat
      c_c_B <- mat_B_rel[c_idx, , drop=FALSE] %*% F_mat
      null_pos_B[i, s] <- sqrt(sum((c_c_B - c_r_B)^2))
      topo_r_B <- .calc_core_topology(mat_B_rel[r_idx, ], F_mat)
      topo_c_B <- .calc_core_topology(mat_B_rel[c_idx, ], F_mat)
      null_fdis_B[i, s] <- topo_c_B$FDis - topo_r_B$FDis
      # FRic under Model B: not evaluated (incidence fixed -> hull invariant)

      # --- MODEL C: IDENTITY (Trait Shuffle) ---
      c_r_C <- M[r_idx, , drop=FALSE] %*% F_mat_shuf
      c_c_C <- M[c_idx, , drop=FALSE] %*% F_mat_shuf
      null_pos_C[i, s] <- sqrt(sum((c_c_C - c_r_C)^2))
      topo_r_C <- .calc_core_topology(as.numeric(M[r_idx, ]), F_mat_shuf)
      topo_c_C <- .calc_core_topology(as.numeric(M[c_idx, ]), F_mat_shuf)
      null_fdis_C[i, s] <- topo_c_C$FDis - topo_r_C$FDis
      null_fric_C[i, s] <- topo_c_C$FRic - topo_r_C$FRic
    }
  }

  # Evaluation helper with robust sd guard
  eval_null <- function(obs, null_dist, two_tailed = FALSE) {
    if (all(is.na(null_dist))) return(list(SES = NA_real_, P = NA_real_))
    null_dist_clean <- null_dist[!is.na(null_dist)]
    if (length(null_dist_clean) < 2) return(list(SES = NA_real_, P = NA_real_))
    sd_val <- stats::sd(null_dist_clean)
    if (sd_val < 1e-12) return(list(SES = NaN, P = 1.000))
    ses <- (obs - mean(null_dist_clean)) / sd_val
    n_eff <- length(null_dist_clean)
    pval <- if(two_tailed) {
      (sum(abs(null_dist_clean) >= abs(obs)) + 1) / (n_eff + 1)
    } else {
      (sum(null_dist_clean >= obs) + 1) / (n_eff + 1)
    }
    return(list(SES = ses, P = pval))
  }

  df_res <- data.frame()
  for(s in entities) {
    obs_pos <- if(!is.null(x$site_results[[s]]$abs_dist)) x$site_results[[s]]$abs_dist else x$pairwise_results$Delta_C_abs[x$pairwise_results$Community_A == strsplit(s, "_vs_")[[1]][1] & x$pairwise_results$Community_B == strsplit(s, "_vs_")[[1]][2]]
    obs_fdis <- if(!is.null(x$site_results[[s]]$Delta_FDis)) x$site_results[[s]]$Delta_FDis else x$pairwise_results$Delta_FDis[x$pairwise_results$Community_A == strsplit(s, "_vs_")[[1]][1] & x$pairwise_results$Community_B == strsplit(s, "_vs_")[[1]][2]]
    obs_fric <- if(!is.null(x$site_results[[s]]$Delta_FRic)) x$site_results[[s]]$Delta_FRic else x$pairwise_results$Delta_FRic[x$pairwise_results$Community_A == strsplit(s, "_vs_")[[1]][1] & x$pairwise_results$Community_B == strsplit(s, "_vs_")[[1]][2]]

    e_pos_A <- eval_null(obs_pos, null_pos_A[, s], FALSE)
    e_pos_B <- eval_null(obs_pos, null_pos_B[, s], FALSE)
    e_pos_C <- eval_null(obs_pos, null_pos_C[, s], FALSE)

    e_fdis_A <- eval_null(obs_fdis, null_fdis_A[, s], TRUE)
    e_fdis_B <- eval_null(obs_fdis, null_fdis_B[, s], TRUE)
    e_fdis_C <- eval_null(obs_fdis, null_fdis_C[, s], TRUE)

    e_fric_A <- eval_null(obs_fric, null_fric_A[, s], TRUE)
    # FRic under Model B: bypass (not applicable)
    e_fric_B <- list(SES = NA_real_, P = NA_real_)
    e_fric_C <- eval_null(obs_fric, null_fric_C[, s], TRUE)

    df_res <- rbind(df_res,
                    data.frame(Contrast = s, Filter = "Structural", Metric = "Position (Delta C)", Observed = round(obs_pos, 4), SES = round(e_pos_A$SES, 2), P_value = round(e_pos_A$P, 3)),
                    data.frame(Contrast = s, Filter = "Quantitative", Metric = "Position (Delta C)", Observed = round(obs_pos, 4), SES = round(e_pos_B$SES, 2), P_value = round(e_pos_B$P, 3)),
                    data.frame(Contrast = s, Filter = "Identity", Metric = "Position (Delta C)", Observed = round(obs_pos, 4), SES = round(e_pos_C$SES, 2), P_value = round(e_pos_C$P, 3)),

                    data.frame(Contrast = s, Filter = "Structural", Metric = "Dispersion (Delta FDis)", Observed = round(obs_fdis, 4), SES = round(e_fdis_A$SES, 2), P_value = round(e_fdis_A$P, 3)),
                    data.frame(Contrast = s, Filter = "Quantitative", Metric = "Dispersion (Delta FDis)", Observed = round(obs_fdis, 4), SES = round(e_fdis_B$SES, 2), P_value = round(e_fdis_B$P, 3)),
                    data.frame(Contrast = s, Filter = "Identity", Metric = "Dispersion (Delta FDis)", Observed = round(obs_fdis, 4), SES = round(e_fdis_C$SES, 2), P_value = round(e_fdis_C$P, 3)),

                    data.frame(Contrast = s, Filter = "Structural", Metric = "Volume (Delta FRic)", Observed = round(obs_fric, 4), SES = round(e_fric_A$SES, 2), P_value = round(e_fric_A$P, 3)),
                    data.frame(Contrast = s, Filter = "Quantitative", Metric = "Volume (Delta FRic)", Observed = round(obs_fric, 4), SES = round(e_fric_B$SES, 2), P_value = round(e_fric_B$P, 3)),
                    data.frame(Contrast = s, Filter = "Identity", Metric = "Volume (Delta FRic)", Observed = round(obs_fric, 4), SES = round(e_fric_C$SES, 2), P_value = round(e_fric_C$P, 3))
    )
  }
  x$null_models <- df_res
  return(x)
}

#' Identify Functional Entities (Species Clustering)
#'
#' @description
#' Clusters species into discrete functional entities based on shared
#' morphological, physiological, or ecological traits.
#'
#' @param traits A data frame or matrix of functional traits.
#' @param dist_method Character. Distance metric. Default is \code{"gower"} to handle mixed data types.
#' @param hclust_method Character. Agglomeration method for hierarchical clustering. Default is \code{"ward.D2"}.
#' @param k Integer. Desired number of functional entities (clusters). If \code{NULL}, \code{h} must be provided.
#' @param h Numeric. Height at which to cut the dendrogram. If both \code{k} and \code{h} are \code{NULL}, defaults to \code{k = 3}.
#'
#' @return An S3 object of class \code{ascfcd_entities}.
#'
#' @examples
#' traits <- data.frame(
#'   Mass = c(15, 30, 60, 150, 400),
#'   Beak = c(10, 15, 28,  45,  85),
#'   Diet = factor(c(0, 1, 1, 1, 0))
#' )
#' rownames(traits) <- paste0("Sp", 1:5)
#'
#' ent <- asc_entities(traits, k = 2)
#' summary(ent)
#'
#' @export
asc_entities <- function(traits, dist_method = "gower", hclust_method = "ward.D2", k = NULL, h = NULL) {

  if (!requireNamespace("cluster", quietly = TRUE)) {
    stop("The 'cluster' package is required for distance calculations.")
  }

  # 1. Cálculo de Distancias
  d_mat <- cluster::daisy(traits, metric = dist_method)

  # 2. Clustering Jerárquico
  hc <- stats::hclust(d_mat, method = hclust_method)

  # 3. Determinación del corte (agrupamiento)
  if (is.null(k) && is.null(h)) {
    warning("Neither 'k' (number of clusters) nor 'h' (cut height) were provided. Defaulting to k = 3 functional entities.")
    k <- 3
  }

  clusters <- stats::cutree(hc, k = k, h = h)

  # 4. Ensamblaje de resultados
  res_df <- data.frame(
    Species = rownames(traits),
    Entity_ID = paste0("Entity_", clusters),
    stringsAsFactors = FALSE
  )

  # Añadimos los rasgos originales para facilitar la inspección del usuario
  res_df <- cbind(res_df, traits)

  output <- list(
    entity_classification = res_df,
    distance_matrix = d_mat,
    hclust_obj = hc,
    method = hclust_method,
    metric = dist_method
  )

  class(output) <- "ascfcd_entities"
  return(output)
}

---
title: "ascent: A multi-layer framework for decomposing functional community restructuring into positional, dispersive and boundary components under hierarchical null models."
author: "Rogelio R. Muñoz-Li & Flavia Alvarez-Denis"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ascent: A multi-layer framework for decomposing functional community restructuring into positional, dispersive and boundary components under hierarchical null models.}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE,
  fig.width = 10,
  fig.height = 5.5
)
```

## Part I - Conceptual Framework

The `ascent` package implements the **ASC-CFD** (Assemblage Shift Characterization — Community Functional Dynamics) framework. Communities can exhibit identical levels of functional turnover while following fundamentally different ecological trajectories. One community may experience a directional shift in its functional equilibrium, another may simply reorganize biomass internally, and a third may expand into previously unoccupied regions of trait space. Traditional metrics often collapse these distinct processes into a single value. `ascent` instead uses a multi-layer topological approach to decompose community restructuring into three orthogonal geometric components:

1. **Positional Shift ($r\Delta C_{\%}$):** The net directional displacement of the community centroid.
2. **Dispersive Shift ($\Delta FDis$):** The internal demographic reorganization (expansion or contraction of niche variance).
3. **Boundary Shift ($\Delta FRic$):** The multidimensional Convex Hull volume (expansion or collapse of extreme phenotypes).


**Table 1**. Functional diversity metrics used in ASC-CFD frameworks.

| Layer      | Metric | Ecological Question                                |
| ---------- | ------ | -------------------------------------------------- |
| Position   | ΔC     | Where is the community moving?                     |
| Dispersion | ΔFDis  | How is biomass being redistributed?                |
| Boundary   | ΔFRic  | Are functional boundaries expanding or collapsing? |


To isolate deterministic environmental filtering from stochastic noise, `ascent` triggers a hierarchical triad of null models: **Structural** (Incidence), **Quantitative** (Demographic), and **Identity** (Trait Shuffle).

### Simulating an Avian Metacommunity
We simulate a bird community experiencing habitat degradation. Functional matrices often contain mixed data types. Here, we combine quantitative traits (body mass, beak length) and binary traits (diet, habitat specialization). Binary traits MUST be encoded as `factor` to correctly apply the **Gower** distance metric.

```{r data_simulation}
library(ascent)

# 1. Functional Trait Space
set.seed(42)
aves_traits <- data.frame(
  Body_Mass   = c(15, 20, 30, 45, 60, 90, 150, 250, 400, 600),
  Beak_Length = c(10, 12, 15, 22, 28, 35,  45,  60,  85, 120),
  Frugivore   = factor(c(0, 0, 1, 1, 1, 0,  1,  1,  1,  0)),
  Forest_Dep  = factor(c(0, 0, 0, 0, 1, 1,  1,  1,  1,  1))
)
rownames(aves_traits) <- paste0("Av_sp", 1:10)

# 2a. Temporal Matrix (Deforestation)
abund_time <- rbind(
  Site1_Ref = c( 0,  0, 10, 15, 25, 20, 10, 8, 4, 2), # Mature forest
  Site1_Imp = c(40, 35, 15,  5,  0,  0,  0, 0, 0, 0)  # Logged area
)

# 2b. Spatial Matrix (Landscape Gradient)
abund_space <- rbind(
  Primary   = c( 0,  0,  5, 10, 20, 25, 15, 10, 5, 2),
  Secondary = c( 0,  5, 10, 20, 35, 20, 10,  0, 0, 0),
  Agri      = c( 0,  0,  0,  5, 45, 45,  5,  0, 0, 0)
)
```

---

## Part II - Temporal Restructuring

We start with the core engine: evaluating the functional restructuring driven by deforestation in Site 1 using `asc_paired()`. We simultaneously trigger the triad of null models using `asc_null()`. *(Note: `n_perm = 99` is used for CRAN vignette compilation speed. Use 999 or 9999 for research).*

```{r temporal_shift}
res_paired <- asc_paired(
  traits = aves_traits, abund = abund_time, 
  sites = c("Site1", "Site1"), time = c("Reference", "Impacted"),
  ref_time = "Reference", dist_method = "gower"
)
res_paired <- asc_null(res_paired, n_perm = 99, seed = 123)
summary(res_paired)
```

**Ecological Interpretation:** The community experienced a displacement of the functional centroid (42.42% of the regional functional diameter), while changes in dispersion and volume were small and consistent with random expectations. The only component that exceeded the null models was position under the identity filter, indicating that the observed restructuring was associated with a non-random selection of combinations of functional traits. Consequently, the results suggest a directional functional filtering process without evidence of significant contraction of the functional niche or internal reorganization of biomass.

---

## Part III - Species Leverage

To dissect the biological drivers behind this geometric shift, we extract the **Functional Leverage**. This algorithm projects the multidimensional demographic shift ($\Delta p_i$) of each species onto the unit directional vector of the ecosystem.

```{r leverage}
# Extract the specific topological drivers
drivers_time <- asc_transitions(res_paired)
head(drivers_time$Site1$species_leverage, 4)

# Plot the PCoA trajectory and the leverage divergence
plot(res_paired, contrast = "Site1", type = "both", n_sp = 5)
```

**Ecological Interpretation:** Species Av_sp1 (small, open-area generalist) exhibited the highest positive leverage, indicating that its demographic expansion strongly pulled the community centroid toward the degraded state. In contrast, Av_sp6 showed negative leverage because it declined despite occupying a position aligned with the direction of change, partially opposing the overall displacement. Conversely, the decline of Av_sp5 generated a small positive leverage effect by removing biomass from a region of trait space located opposite to the observed trajectory, thereby facilitating the centroid shift.

---

## Part IV - Spatial Networks

For landscape ecology or beta-diversity studies, `ascent` computes the bidirectional spatial divergence between all possible community pairs using `asc_pairwise()`.

```{r spatial_network}
res_pw <- asc_pairwise(traits = aves_traits, abund = abund_space, dist_method = "gower")
res_pw <- asc_null(res_pw, n_perm = 99, seed = 42)
summary(res_pw)

# Visualize the functional gap between Primary and Secondary Forest
plot(res_pw, contrast = "Primary_vs_Secondary", type = "both", n_sp = 5)

# Visualize the severe functional gap between Secondary Forest and Agriculture
plot(res_pw, contrast = "Secondary_vs_Agri", type = "both", n_sp = 5)
```

**Ecological Interpretation:** The spatial analysis revealed relatively small functional differences among communities. None of the observed shifts exceeded the expectations generated by the hierarchical null models, suggesting that the apparent variation among habitats can be explained by stochastic fluctuations in species composition and abundance. In this simulated example, agricultural intensification produced only modest changes in community topology, resulting in limited displacement of the functional centroid and minor reductions in functional volume. This example illustrates how ASC-CFD can distinguish between apparent community differences and statistically supported functional restructuring. Although the communities differ in composition, none of the observed topological shifts were stronger than expected under the null models, indicating weak evidence for deterministic functional filtering.

---

## Part V - Baseline Topology

Sometimes, a researcher only needs to report the absolute structural metrics of isolated communities without computing directional networks. `asc_baseline()` calculates the absolute functional topology of isolated communities.

```{r baseline}
base_topo <- asc_baseline(traits = aves_traits, abund = abund_space, dist_method = "gower")
summary(base_topo)
```

The baseline metrics reveal a gradual reduction in both functional richness and functional dispersion from `Primary` forest to `Agri` habitat, indicating that agricultural communities occupy a somewhat smaller and less dispersed region of the functional space.

---

## Part VI - Functional Entities

As a supplementary utility, the package provides `asc_entities()`, an algorithm that clusters the regional species pool into discrete functional entities based on morphological and ecological trait distances, regardless of spatial abundance.

```{r entities, fig.width=7, fig.height=5}
# Cluster the species into 3 functional entities using Gower distance
func_ent <- asc_entities(traits = aves_traits, dist_method = "gower", k = 3)
summary(func_ent)

# Visualize the functional dendrogram
plot(func_ent)
```

This classification helps researchers map continuous traits into discrete ecological guilds prior to mapping community-level dynamics.

**Choosing the Appropriate Function**

**Table 2**. Functions included in `ascent` package and its purposes.

| Objective              | Function            |
| ---------------------- | ------------------- |
| Temporal restructuring | `asc_paired()`      |
| Spatial divergence     | `asc_pairwise()`    |
| Species drivers        | `asc_transitions()` |
| Baseline topology      | `asc_baseline()`    |
| Functional entities    | `asc_entities()`    |
| Null model inference   | `asc_null()`        |

---

## Part VII - Methodological Notes

### Interpreting FRic ($\Delta FRic$)

FRic is computed as the volume of the convex hull in the retained PCoA axes. It captures the **outer boundary** of the functional space and has the following properties:

- It is driven exclusively by **extreme species** (hull vertices). Internal species, regardless of abundance, do not alter FRic.
- It is **not abundance-weighted.** A species that increases from 1% to 90% will not change FRic unless it lies on the hull boundary.
- When a community has fewer species than retained axes ($S < k$), the hull is geometrically undefined. In this case, ascent reports `FRic = NA` rather than `FRic = 0`, because a zero-volume hull is a meaningful geometric statement (collinear species), while `NA` signals that the metric cannot be computed.

**Recommendation:** Interpret $\Delta FRic$ as a topological descriptor of the functional boundary, complementary to (but not substituting for) the abundance-weighted $\Delta FDis$.

### Normalization and Cross-Study Comparisons

- **$r\Delta C_{\%}$** (positional shift) is normalized by the regional functional diameter ($D_{max,regional}$) and is directly comparable across studies.
- **$\Delta FDis$** and **$\Delta FRic$** are absolute differences in the retained PCoA space. Their magnitude depends on the number of retained axes, the trait set, and the Gower distance matrix. Direct comparison across studies with different trait pools requires caution.

### Null Model Scope

`asc_null()` operates on the full stacked community matrix and assumes a **shared regional species pool.** The curveball algorithm permutes species across all rows simultaneously. For biogeographically independent sites, run `asc_null()` on each contrast separately.

Model A assigns **uniform relative abundances** ($1/S_{local}$) to all species present after the curveball permutation. This means the null distribution for FDis under Model A tests whether the shift is extreme given random species composition with equitable abundances, not with the observed SAD.

### Functional Leverage: Additive Decomposition

The Functional Leverage satisfies a strict mathematical identity:

$$\sum_{i=1}^{S} \text{Leverage}_i = \|\Delta C\|$$

Each species' leverage is the product of its demographic change and its alignment with the centroid trajectory:

$$\text{Leverage}_i = \Delta p_i \cdot \text{proj}(\mathbf{f}_i, \hat{v})$$

where $\Delta p_i = p_{i,\text{comp}} - p_{i,\text{ref}}$ is the change in relative abundance, and $\text{proj}(\mathbf{f}_i, \hat{v})$ is the scalar projection of the species' PCoA coordinates onto the unit directional vector $\hat{v}$ of the centroid shift. Species with positive leverage **drive** the shift; species with negative leverage **resist** it.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.ascfcd_transitions}
\alias{summary.ascfcd_transitions}
\title{Summary for Functional Leverage Analysis}
\usage{
\method{summary}{ascfcd_transitions}(object, n_sp = 5, ...)
}
\arguments{
\item{object}{An object of class \code{ascfcd_transitions}.}

\item{n_sp}{Integer. Number of top species to display per contrast. Default is 5.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the object.
}
\description{
Provides an overview of the species leverage for each contrast.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.ascfcd_base}
\alias{summary.ascfcd_base}
\title{Summary for Baseline Functional Entities}
\usage{
\method{summary}{ascfcd_base}(object, ...)
}
\arguments{
\item{object}{An object of class \code{ascfcd_base}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the entities results data frame.
}
\description{
Provides an overview of the baseline functional topology of the evaluated entities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_pairwise.R
\name{asc_pairwise}
\alias{asc_pairwise}
\title{Pairwise Functional Spatial Divergence}
\usage{
asc_pairwise(
  traits,
  abund,
  dist_method = "gower",
  dim_retention = c("variance", "broken_stick"),
  var_tol = 0.8,
  na.rm = TRUE
)
}
\arguments{
\item{traits}{A data frame or matrix of functional traits.}

\item{abund}{A data frame or matrix of species abundances.}

\item{dist_method}{Distance metric for the trait matrix. Default is "gower".}

\item{dim_retention}{Character. Method for dimensionality reduction.}

\item{var_tol}{Numeric. Proportion of cumulative variance to retain. Default is 0.80.}

\item{na.rm}{Logical. Remove missing values? Default is TRUE.}
}
\value{
An S3 object of class \code{ascfcd_pw}.
}
\description{
Calculates spatial functional divergence across three layers: position (Delta C), dispersion (FDis), and volume (FRic).
}
\note{
\strong{Leverage direction:} In pairwise mode, leverage is computed from
Community_A toward Community_B. Reversing the pair order reverses the
leverage signs. The direction is determined by alphabetical ordering of
community names.

\strong{Normalization:} Delta FDis and Delta FRic are absolute differences (see
\code{\link{asc_paired}} for details).
}
\examples{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85)
)
rownames(traits) <- paste0("Sp", 1:5)

abund <- rbind(
  Forest  = c(0,  5, 25, 20, 10),
  Field   = c(30, 20, 10,  0,  0),
  Wetland = c(5, 10, 15, 15,  5)
)

res_pw <- asc_pairwise(traits, abund, dist_method = "euclidean")
summary(res_pw)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_baseline.R
\name{asc_baseline}
\alias{asc_baseline}
\title{Baseline Functional Topology of Entities}
\usage{
asc_baseline(
  traits,
  abund,
  dist_method = "gower",
  dim_retention = c("variance", "broken_stick"),
  var_tol = 0.8,
  na.rm = TRUE
)
}
\arguments{
\item{traits}{A data frame or matrix of functional traits.}

\item{abund}{A data frame or matrix of species abundances.}

\item{dist_method}{Distance metric for the trait matrix. Default is "gower".}

\item{dim_retention}{Character. Method for dimensionality reduction. Options are \code{"variance"} or \code{"broken_stick"}.}

\item{var_tol}{Numeric. Proportion of cumulative variance to retain. Default is 0.80.}

\item{na.rm}{Logical. Remove missing values? Default is TRUE.}
}
\value{
An S3 object of class \code{ascfcd_base}.
}
\description{
Calculates the absolute functional centroid (CWM), functional dispersion (FDis),
and convex hull volume (FRic) for each community or site independently.
}
\examples{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85)
)
rownames(traits) <- paste0("Sp", 1:5)

abund <- rbind(
  Forest = c(0,  5, 25, 20, 10),
  Field  = c(30, 20, 10,  0,  0)
)

base <- asc_baseline(traits, abund, dist_method = "euclidean")
summary(base)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_functional_space.R
\name{assess_functional_space}
\alias{assess_functional_space}
\title{Assess Functional Space Quality}
\usage{
assess_functional_space(
  traits,
  dist_method = "gower",
  dim_retention = c("variance", "broken_stick"),
  var_tol = 0.8
)
}
\arguments{
\item{traits}{A data frame or matrix of functional traits.}

\item{dist_method}{Distance metric for the trait matrix. Default is \code{"gower"}.}

\item{dim_retention}{Character. Method for dimensionality reduction.
Options are \code{"variance"} or \code{"broken_stick"}. Default is \code{"variance"}.}

\item{var_tol}{Numeric. Proportion of cumulative variance to retain (used
when \code{dim_retention = "variance"}). Default is 0.80.}
}
\value{
A list of class \code{ascfcd_space} with:
\describe{
\item{quality}{Numeric. PCoA quality index (\code{sum(positive_eig) / sum(abs(all_eig))}). Values above 0.80 are generally acceptable.}
\item{n_species}{Integer. Number of species in the trait matrix.}
\item{n_traits}{Integer. Number of traits.}
\item{k_retained}{Integer. Number of PCoA axes retained.}
\item{var_retained}{Numeric. Cumulative variance explained by retained axes.}
\item{axis_var}{Numeric vector. Relative variance per retained axis.}
\item{n_neg_eigenvalues}{Integer. Count of negative eigenvalues.}
\item{neg_eigenvalue_pct}{Numeric. Percentage of total absolute eigenvalue represented by negative eigenvalues.}
\item{eigenvalues}{Numeric vector. All raw eigenvalues from PCoA.}
}
}
\description{
Diagnostic tool to evaluate the quality of the PCoA-based functional space
constructed from a trait matrix. Intended for use before running core
analyses (\code{asc_paired}, \code{asc_pairwise}, etc.) to identify
potential distortion from negative eigenvalues or excessive dimensionality
reduction.
}
\details{
The Gower distance is not strictly Euclidean, which means PCoA may produce
negative eigenvalues. This function reports the full eigenvalue spectrum
and computes a quality index so users can decide whether the functional
space is an adequate representation of the original distance matrix.

A quality below 0.80 indicates that negative eigenvalues represent more
than 20\\% of the total variation. In such cases, consider:
\itemize{
\item Reviewing the trait data for problematic variables.
\item Applying a Cailliez or Lingoes correction to the distance matrix before ordination.
\item Using a different distance metric (e.g., \code{"euclidean"} for purely continuous traits).
}
}
\examples{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85),
  Diet = factor(c(0, 1, 1, 1, 0))
)
rownames(traits) <- paste0("Sp", 1:5)

diag <- assess_functional_space(traits)
diag

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.ascfcd}
\alias{summary.ascfcd}
\title{Summary for Paired Functional Centroid Displacement}
\usage{
\method{summary}{ascfcd}(object, ...)
}
\arguments{
\item{object}{An object of class \code{ascfcd}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns a data frame with the displacement metrics.
}
\description{
Provides a concise, multi-layer overview of the paired ASC-CFD analysis,
including null models and top functional drivers.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.ascfcd}
\alias{plot.ascfcd}
\title{Plot Multidimensional Functional Restructuring}
\usage{
\method{plot}{ascfcd}(x, contrast, type = c("both", "pcoa", "leverage"), n_sp = 10, ...)
}
\arguments{
\item{x}{An object of class \code{ascfcd}.}

\item{contrast}{Character. The exact name of the site/contrast to plot.}

\item{type}{Character. What to plot? \code{"pcoa"} (Trajectory & Hull), \code{"leverage"} (Drivers), or \code{"both"}. Default is \code{"both"}.}

\item{n_sp}{Integer. Number of top species to display in the leverage plot. Default is 10.}

\item{...}{Additional graphical arguments.}
}
\value{
A ggplot object (or patchwork object if \code{type = "both"}).
}
\description{
Visualizes the functional trajectory (Centroid shift) and volume shift (Convex Hull)
of a specific site/contrast in the PCoA space, alongside the species leverage.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.ascfcd_entities}
\alias{summary.ascfcd_entities}
\title{Summary for Functional Entities Classification}
\usage{
\method{summary}{ascfcd_entities}(object, ...)
}
\arguments{
\item{object}{An object of class \code{ascfcd_entities}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the classification data frame.
}
\description{
Provides an overview of the species clustering into functional entities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_transitions.R
\name{asc_transitions}
\alias{asc_transitions}
\title{Extract Functional Transitions and Species Leverage}
\usage{
asc_transitions(x, ...)
}
\arguments{
\item{x}{An object of class \code{ascfcd} or \code{ascfcd_pw}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An S3 object of class \code{ascfcd_transitions}.
}
\description{
Calculates the multidimensional Functional Leverage of each species,
quantifying their direct contribution to the positional shift (Layer 1 - Delta C)
of the ecosystem.
}
\examples{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85)
)
rownames(traits) <- paste0("Sp", 1:5)

abund <- rbind(
  Ref = c(0, 10, 25, 20, 5),
  Imp = c(30, 15,  5,  0, 0)
)

res <- asc_paired(
  traits, abund,
  sites = c("S1", "S1"),
  time = c("R", "I"), ref_time = "R",
  dist_method = "euclidean"
)
drivers <- asc_transitions(res)
drivers$S1$species_leverage

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.ascfcd_pw}
\alias{plot.ascfcd_pw}
\title{Plot Pairwise Functional Spatial Divergence}
\usage{
\method{plot}{ascfcd_pw}(x, contrast, type = c("both", "pcoa", "leverage"), n_sp = 10, ...)
}
\arguments{
\item{x}{An object of class \code{ascfcd_pw}.}

\item{contrast}{Character. The contrast to plot (e.g., "SiteA_vs_SiteB").}

\item{type}{Character. What to plot? \code{"pcoa"}, \code{"leverage"}, or \code{"both"}.}

\item{n_sp}{Integer. Number of top species to display. Default is 10.}

\item{...}{Additional graphical arguments.}
}
\value{
A ggplot object.
}
\description{
Visualizes spatial functional restructuring (Centroid & Hull) between two communities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_paired.R
\name{asc_paired}
\alias{asc_paired}
\title{Paired Functional Multidimensional Restructuring}
\usage{
asc_paired(
  traits,
  abund,
  sites,
  time,
  ref_time,
  dist_method = "gower",
  dim_retention = c("variance", "broken_stick"),
  var_tol = 0.8,
  na.rm = TRUE
)
}
\arguments{
\item{traits}{A data frame or matrix of functional traits.}

\item{abund}{A data frame or matrix of species abundances.}

\item{sites}{A vector indicating the site identifier.}

\item{time}{A vector indicating the temporal state.}

\item{ref_time}{A character string specifying the reference state.}

\item{dist_method}{Distance metric for the trait matrix. Default is "gower".}

\item{dim_retention}{Character. Method for dimensionality reduction.}

\item{var_tol}{Numeric. Proportion of cumulative variance to retain. Default is 0.80.}

\item{na.rm}{Logical. Remove missing values? Default is TRUE.}
}
\value{
An S3 object of class \code{ascfcd}.
}
\description{
Calculates centroid displacement (Layer 1), dispersion shift (Layer 2 - FDis),
and volume shift (Layer 3 - FRic) between paired states.
}
\note{
\strong{Normalization:} Delta FDis and Delta FRic are absolute differences, not
normalized by the regional pool. Values are not directly comparable across
studies with different species pools or trait scales. Only rDelta_C (position)
is normalized by Dmax_regional.

\strong{FRic = NA:} When a community has fewer species than retained PCoA axes,
the convex hull is geometrically undefined and FRic is set to \code{NA},
which propagates to Delta_FRic.

\strong{Temporal replicates:} When multiple rows share the same site and temporal
state (e.g., three reference plots), their relative abundances are averaged
(column means) before computing the centroid. This is equivalent to treating
replicates as a single pooled community. Intra-state variability is not
propagated to downstream metrics.
}
\examples{
# Simulate deforestation impact on a bird community
traits <- data.frame(
  Mass  = c(15, 30, 60, 150, 400),
  Beak  = c(10, 15, 28,  45,  85),
  Diet  = factor(c(0, 1, 1, 1, 0))
)
rownames(traits) <- paste0("Sp", 1:5)

abund <- rbind(
  Reference = c(0, 10, 25, 20, 5),
  Impacted  = c(30, 15,  5,  0, 0)
)

res <- asc_paired(
  traits, abund,
  sites = c("Site1", "Site1"),
  time = c("Reference", "Impacted"),
  ref_time = "Reference"
)
summary(res)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_entities.R
\name{asc_entities}
\alias{asc_entities}
\title{Identify Functional Entities (Species Clustering)}
\usage{
asc_entities(
  traits,
  dist_method = "gower",
  hclust_method = "ward.D2",
  k = NULL,
  h = NULL
)
}
\arguments{
\item{traits}{A data frame or matrix of functional traits.}

\item{dist_method}{Character. Distance metric. Default is \code{"gower"} to handle mixed data types.}

\item{hclust_method}{Character. Agglomeration method for hierarchical clustering. Default is \code{"ward.D2"}.}

\item{k}{Integer. Desired number of functional entities (clusters). If \code{NULL}, \code{h} must be provided.}

\item{h}{Numeric. Height at which to cut the dendrogram. If both \code{k} and \code{h} are \code{NULL}, defaults to \code{k = 3}.}
}
\value{
An S3 object of class \code{ascfcd_entities}.
}
\description{
Clusters species into discrete functional entities based on shared
morphological, physiological, or ecological traits.
}
\examples{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85),
  Diet = factor(c(0, 1, 1, 1, 0))
)
rownames(traits) <- paste0("Sp", 1:5)

ent <- asc_entities(traits, k = 2)
summary(ent)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.ascfcd_pw}
\alias{summary.ascfcd_pw}
\title{Summary for Pairwise Functional Spatial Divergence}
\usage{
\method{summary}{ascfcd_pw}(object, ...)
}
\arguments{
\item{object}{An object of class \code{ascfcd_pw}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the pairwise results data frame.
}
\description{
Provides a comprehensive overview of the spatial network ASC-CFD analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.ascfcd_entities}
\alias{plot.ascfcd_entities}
\title{Plot Functional Entities Dendrogram}
\usage{
\method{plot}{ascfcd_entities}(x, ...)
}
\arguments{
\item{x}{An object of class \code{ascfcd_entities}.}

\item{...}{Additional graphical arguments passed to \code{plot.hclust}.}
}
\value{
No return value, called for side effects (plots a dendrogram).
}
\description{
Plots the hierarchical clustering dendrogram of the functional entities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc_null.R
\name{asc_null}
\alias{asc_null}
\title{Multi-Level Null Models for Multidimensional Functional Shifts}
\usage{
asc_null(x, n_perm = 999, seed = NULL)
}
\arguments{
\item{x}{An object of class \code{ascfcd} or \code{ascfcd_pw}.}

\item{n_perm}{Integer. Number of permutations. Default is 999.}

\item{seed}{Integer. Random seed for reproducibility.}
}
\value{
The original object with an appended \code{null_models} data frame.
}
\description{
Evaluates the statistical significance of functional shifts using three null models:
Structural (Curveball), Quantitative (SAD Reshuffle), and Identity (Trait Shuffle).
}
\note{
\strong{Model A (Structural):} After curveball permutation, all species present
receive uniform relative abundance (1/S_local). This means the null distribution
for FDis conflates the effect of taxonomic identity with the assumption of
equitability. The SES tests whether the observed shift is extreme given
random species composition, \emph{not} given random composition with the observed SAD.

\strong{Model B (Quantitative):} Because incidence is held fixed, the convex hull
is invariant across permutations. Delta FRic under Model B is reported as
\code{NA} (not applicable), not zero.

\strong{Model C (Identity):} Statistical power is limited when the regional species
pool is small (< 15 species). With few species, the number of unique trait
permutations is small, reducing the resolution of the null distribution.
Consider increasing \code{n_perm} and interpreting marginal p-values
(0.05 < p < 0.10) with caution.

\strong{Scope:} When multiple contrasts are evaluated simultaneously, the curveball
operates on the full stacked matrix, assuming a shared regional species pool.
For biogeographically independent sites, run \code{asc_null()} on each
contrast separately.
}
\examples{
\donttest{
traits <- data.frame(
  Mass = c(15, 30, 60, 150, 400),
  Beak = c(10, 15, 28,  45,  85),
  Diet = factor(c(0, 1, 1, 1, 0))
)
rownames(traits) <- paste0("Sp", 1:5)

abund <- rbind(
  Reference = c(0, 10, 25, 20, 5),
  Impacted  = c(30, 15,  5,  0, 0)
)

res <- asc_paired(
  traits, abund,
  sites = c("S1", "S1"),
  time = c("Reference", "Impacted"),
  ref_time = "Reference"
)
res <- asc_null(res, n_perm = 99, seed = 42)
res$null_models
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.ascfcd_transitions}
\alias{plot.ascfcd_transitions}
\title{Plot Functional Leverage}
\usage{
\method{plot}{ascfcd_transitions}(x, contrast, n_sp = 10, ...)
}
\arguments{
\item{x}{An object of class \code{ascfcd_transitions}.}

\item{contrast}{Character. The contrast to plot.}

\item{n_sp}{Integer. Number of top species to display. Default is 10.}

\item{...}{Additional graphical arguments.}
}
\value{
A ggplot object.
}
\description{
Visualizes the top species driving functional centroid displacement.
}
